@@ -62,11 +62,12 @@ enum {
   LONG_OPT_SAVE_UPROBES,
   LONG_OPT_TARGET_NAMESPACES,
   LONG_OPT_MONITOR,
+  LONG_OPT_INTERACTIVE,
 };
 
 // NB: when adding new options, consider very carefully whether they
 // should be restricted from stap clients (after --client-options)!
-#define STAP_SHORT_OPTIONS "hVvtp:I:e:E:o:R:r:a:m:kgPc:x:D:bs:uqiwl:d:L:FS:B:J:jWG:"
+#define STAP_SHORT_OPTIONS "hVvtp:I:e:E:o:R:r:a:m:kgPc:x:D:bs:uqiwl:d:L:FS:B:J:jWG:T:"
 
 extern struct option stap_long_options[];
 
