@@ -32,6 +32,10 @@ extern "C" {
 #include <readline/readline.h>
 #include <readline/history.h>
 #include <ctype.h>
+
+#if HAVE_LIBSQLITE3
+  #include <sqlite3.h>
+#endif
 }
 
 // FIXME: these declarations don't really belong here.
@@ -51,6 +55,14 @@ forked_semantic_pass (systemtap_session &s, vector<string> &script);
 // Ask user a y-or-n question and return 1 iff answer is yes.  The
 // prompt argument should end in "? ". Note that this is a simplified
 // version of gdb's defaulted_query() function.
+#if HAVE_LIBSQLITE3
+struct metafile {
+  string name;
+  string title;
+  string description;
+  string path;
+};
+#endif
 
 enum query_default { no_default,	// There isn't a default.
 		     default_yes,	// The default is "yes".
@@ -106,7 +118,7 @@ query (const char *prompt, query_default qdefault)
 
       response = readline(_F("%s(%s or %s) ", prompt, y_string,
 			     n_string).c_str());
-      if (response == NULL)		// C-d
+      if (response == NULL)		// EOF
 	{
 	  clog << _F("EOF [assumed %c]\n", def_answer);
 	  retval = def_value;
@@ -149,6 +161,7 @@ public:
   virtual ~cmdopt() { }
   string name;				// command/option name 
   string usage;				// command usage (includes options)
+  vector<string> aliases;
 
   // help_text() returns the help text for a command/option
   virtual string help_text(size_t indent __attribute ((unused))) const
@@ -159,8 +172,22 @@ public:
   // handler() is the code associated with a command/option
   virtual bool handler(systemtap_session &s, vector<string> &tokens,
 		       string &input) = 0;
+
+  bool accept(const string& input) const;
 };
 
+bool cmdopt::accept(const string &input) const
+{
+  if (input == name)
+    return true;
+  for (auto it = aliases.begin(); it != aliases.end(); ++it)
+    {
+      if (input == *it)
+        return true;
+    }
+  return false;
+}
+
 typedef vector<cmdopt*> cmdopt_vector;
 typedef vector<cmdopt*>::const_iterator cmdopt_vector_const_iterator;
 typedef vector<cmdopt*>::iterator cmdopt_vector_iterator;
@@ -273,13 +300,29 @@ public:
   help_cmd()
   {
     name = usage = "help";
-    _help_text = "Print this command list.";
+    aliases = { "h", "?" };
+    _help_text = "Print command list or description of a specific command.";
   }
   bool handler(systemtap_session &s __attribute ((unused)),
 	       vector<string> &tokens __attribute ((unused)),
 	       string &input __attribute ((unused)))
   {
-    interactive_usage();
+    tokens.erase(tokens.begin());
+    if (tokens.size() == 0) {
+        interactive_usage();
+        return false;
+    }
+
+    for (auto it = command_vec.begin();
+	 it != command_vec.end(); ++it)
+      {
+        if ((*it)->accept(tokens[0])) {
+	  cout << (*it)->usage << ": " << (*it)->help_text((*it)->usage.size()) << endl;
+          return false;
+	}
+      }
+
+    cout <<"Undefined command \""<< tokens[0]<< "\". Try \"help\" or \"?\" for a full list of commands." << endl;
     return false;
   }
 };
@@ -290,6 +333,7 @@ public:
   list_cmd()
   {
     name = usage = "list";
+    aliases = { "l" };
     _help_text = "Display the current script.";
   }
   bool handler(systemtap_session &s __attribute ((unused)),
@@ -351,8 +395,8 @@ public:
     bool option_found = false;
     if (tokens.size() < 3)
       {
-	cout << endl << "Invalid command" << endl;
-	interactive_usage();
+	cout  << "Invalid command" << endl;
+	cout << usage << ": " << help_text(0) <<endl;
 	return false;
       }
 
@@ -360,7 +404,7 @@ public:
     for (cmdopt_vector_iterator it = option_vec.begin();
 	 it != option_vec.end(); ++it)
       {
-	if (tokens[1] == (*it)->name)
+	if ((*it)->accept(tokens[1]))
 	{
 	  option_found = true;
 	  (*it)->handler(s, tokens, input);
@@ -370,7 +414,7 @@ public:
     if (!option_found)
       {
 	cout << "Invalid option name" << endl;
-	interactive_usage();
+	cout << usage << ": " << help_text(0) <<endl;
       }
     return false;
   }
@@ -402,7 +446,7 @@ public:
     else if (tokens.size() != 2)
       {
 	cout << endl << "Invalid command" << endl;
-	interactive_usage();
+        cout << usage << ": " << help_text(0) <<endl;
 	return false;
       }
 
@@ -410,7 +454,7 @@ public:
     for (cmdopt_vector_iterator it = option_vec.begin();
 	 it != option_vec.end(); ++it)
       {
-	if (tokens[1] == (*it)->name)
+	if ((*it)->accept(tokens[1]))
 	  {
 	    option_found = true;
 	    (*it)->handler(s, tokens, input);
@@ -419,8 +463,7 @@ public:
       }
     if (!option_found)
       {
-	cout << "Invalid option name" << endl;
-	interactive_usage();
+	cout << "Invalid option name" << endl << "Use \"help set\" for a list of options." <<endl;
       }
     return false;
   }
@@ -432,6 +475,7 @@ public:
   quit_cmd()
   {
     name = usage = "quit";
+    aliases = { "q" };
     _help_text = "Quit systemtap.";
   }
   bool handler(systemtap_session &s __attribute ((unused)),
@@ -448,6 +492,7 @@ public:
   add_cmd()
   {
     name = usage = "add";
+    aliases = { "a" };
     _help_text = "Add a global, probe, or function.";
   }
   bool handler(systemtap_session &s,
@@ -461,8 +506,10 @@ public:
 
     // Skip past the add command itself by removing the 1st token.
     tokens.erase(tokens.begin());
-    if (tokens.size() == 0)
+    if (tokens.size() == 0) {
+      cout << name << ": " << _help_text << endl;
       return false;
+    }
 
     // Find the "add" command text and skip past it.  Note that we're
     // using the "raw" (not tokenized) input, so that if someone was
@@ -490,7 +537,7 @@ public:
 	  {
 	    char *response;
 	    response = readline("stap>>> ");
-	    if (response == NULL)		// C-d
+	    if (response == NULL)		// EOF
 	      {
 		clog << endl;
 		break;
@@ -519,6 +566,7 @@ public:
   delete_cmd()
   {
     name = "delete";
+    aliases = { "d" };
     usage = "delete LINE_NUM_RANGE";
     _help_text = "Delete script line(s) by line numbers. To delete a single line, specify LINE_NUM_RANGE as \"10\". To delete a range of lines, specify LINE_NUM_RANGE as \"12-20\". To delete the entire script, give no argument.";
   }
@@ -639,11 +687,19 @@ public:
   {
     if (tokens.size() != 2)
       {
-	cout << endl << "FILE must be specified." << endl;
-	interactive_usage();
+	cout << "FILE must be specified." << endl;
+        cout << usage << ": " << help_text(0) <<endl;
 	return false;
       }
 
+    if (!script_vec.empty())
+      {
+        if (query("Script exists. Overwrite existing script? ", no_default))
+          script_vec.clear();
+        else
+          return false;
+      }
+
     // Originally, we called stap's parser here to read in the
     // script. However, doing so discards comments, preprocessor
     // directives, and rearranges the script. So, let's just read the
@@ -682,8 +738,8 @@ public:
   {
     if (tokens.size() != 2)
       {
-	cout << endl << "FILE must be specified." << endl;
-	interactive_usage();
+	cout << "FILE must be specified." << endl;
+        cout << usage << ": " << help_text(0) <<endl;
 	return false;
       }
 
@@ -696,7 +752,9 @@ public:
 	return false;
       }
 
-    string script = join(script_vec, "\n");
+    string script;
+    if (!script_vec.empty())
+      script = join(script_vec, "\n");
     f << script << endl;
     f.close();
     return false;
@@ -709,6 +767,7 @@ public:
   run_cmd()
   {
     name = usage = "run";
+    aliases = { "r" };
     _help_text = "Run the current script.";
   }
   bool handler(systemtap_session &s,
@@ -754,6 +813,7 @@ public:
   edit_cmd()
   {
     name = usage = "edit";
+    aliases = { "e" };
     _help_text = "Edit the current script. Uses EDITOR environment variable contents as editor (or ex as default).";
   }
   bool handler(systemtap_session &s,
@@ -844,12 +904,146 @@ public:
 
     (void)fclose(fp);
     (void)unlink(temp_path);
-    if (auto_analyze)
+    if (auto_analyze && !script_vec.empty())
       (void)forked_semantic_pass(s, script_vec);
     return false;
   }
 };
 
+#if HAVE_LIBSQLITE3
+class sample_cmd: public cmdopt
+{
+public:
+  sample_cmd()
+  {
+    name = "sample";
+    usage = "sample STATEMENT";
+    _help_text = "Search and load an example script using criteria specified in STATEMENT. STATEMENT can contain any number of case insensitive keywords seperated by OR, AND or NOT (case sensitive). Wildcards can be postfixed to any number of keywords. STATEMENT can contain anything supported by FTS full-text index queries. Example: sample socket AND tcp* OR disk NOT virtual";
+  }
+  bool handler (systemtap_session &s __attribute ((unused)),
+                vector<string> &tokens __attribute ((unused)),
+                string &input __attribute ((unused)))
+  {
+    if (tokens.size() == 1)
+      {
+        cout << usage << ": " << _help_text << endl;
+        return false;
+      }
+
+    sqlite3* db;
+    int rc;
+    vector <metafile>  metalist;
+
+    string dirstr = PKGDATADIR;
+    dirstr.append("/examples/");
+
+    rc = sqlite3_open((dirstr + "metadatabase.db").c_str(), &db);
+    if (rc)
+      {
+	cout << "Error connecting to meta database. RC: " << rc << endl;
+	return false;
+      }
+
+    string sqlstr = "SELECT name, title, description, path FROM metavirt WHERE metavirt MATCH ?;";
+    sqlite3_stmt* sqlstmt;
+    string tokenstring = tokens[1];
+    for (unsigned it = 2; it < tokens.size(); it++)
+      {
+        tokenstring.append(" " + tokens[it]);
+      }
+
+    rc = sqlite3_prepare_v2(db, sqlstr.c_str(), sqlstr.size() + tokenstring.size(), &sqlstmt, NULL);
+    if (rc)
+      {
+        cout << "Error preparing SQL statement. RC: " << rc << endl;
+        return false;
+      }
+
+    rc = sqlite3_bind_text(sqlstmt, 1, tokenstring.c_str(), tokenstring.size(), 0);
+    if (rc)
+      {
+        cout << "Error binding SQL statement. RC: " << rc << endl;
+        return false;
+      }
+
+    metafile file;
+    while (1)
+      {
+	rc = sqlite3_step(sqlstmt);
+	if (rc == SQLITE_ROW)
+	  {
+	    file.name = (char*)sqlite3_column_text(sqlstmt,0);
+            file.title = (char*)sqlite3_column_text(sqlstmt,1);
+            file.description = (char*)sqlite3_column_text(sqlstmt,2);
+            file.path = (char*)sqlite3_column_text(sqlstmt,3);
+            metalist.push_back(file);
+	  }
+	else if (rc == SQLITE_DONE)
+	  break;
+	else
+	  {
+            cout << "Incomplete statement." << endl;
+	    return false;
+	  }
+      }
+    sqlite3_finalize(sqlstmt);
+
+    if (metalist.size() == 0)
+      {
+        cout << "No sample scripts found using \"" << tokenstring << "\"." << endl;
+	cout << "Note, AND, OR and NOT are case sensitive." << endl;
+        return false;
+      }
+
+    char* response;
+    metafile selection;
+    while (selection.name == "")
+      {
+	cout << "Enter a script name for a description or use q to quit:" << endl;
+        for (unsigned it=0; it < metalist.size();it++)
+	  cout << "  " << metalist[it].name << ":  " << metalist[it].title <<  endl;
+
+	response = readline(_F("%s","stap sample>> ").c_str());
+
+        if (response == NULL) // EOF
+          {
+            cout << endl;
+            return false;
+          }
+        if (*response == 'q')
+          return false;
+
+       for (unsigned i=0; i < metalist.size();i++)
+	 {
+	   if (metalist[i].name.find(response) != string::npos)
+	     selection = metalist[i];
+         }
+         if (selection.name == "")
+	   {
+            cout << "Unknown selection." << endl;
+	    continue;
+           }
+	 cout << endl << "Title: " << selection.title << endl << "Name: " << selection.name << endl;
+	 cout << "Description: " << selection.description << endl;
+         if (!query("Would you like to load this example? ", no_default))
+	   {
+	     selection.name = "";
+	    if (!query("Would you like to select another related script? ", no_default))
+	      return false;
+           }
+      }
+
+    tokens.erase(tokens.begin() + 1, tokens.begin() + tokens.size());
+    auto load = new load_cmd;
+    tokens.push_back(dirstr + selection.path + "/" + selection.name);
+    load -> handler(s, tokens, input);
+
+    sqlite3_close(db);
+    return false;
+  }
+};
+#endif
+
 //
 // Supported options for the "set" and "show" commands.
 // 
@@ -1375,7 +1569,7 @@ interactive_completion(const char *text, int start,
       for (cmdopt_vector_const_iterator it = option_command_vec.begin();
 	   it != option_command_vec.end(); ++it)
       {
-	if ((*it)->name == tokens[0])
+	if ((*it)->accept(tokens[0]))
 	{
 	  matches = rl_completion_matches(text, option_generator);
 	  return matches;
@@ -1625,6 +1819,9 @@ interactive_mode (systemtap_session &s, vector<remote*> targets)
   command_vec.push_back(new load_cmd);
   command_vec.push_back(new save_cmd);
   command_vec.push_back(new run_cmd);
+  #if HAVE_LIBSQLITE3
+    command_vec.push_back(new sample_cmd);
+  #endif
   command_vec.push_back(new set_cmd);
   option_command_vec.push_back(command_vec.back());
   command_vec.push_back(new show_cmd);
@@ -1645,6 +1842,23 @@ interactive_mode (systemtap_session &s, vector<remote*> targets)
   option_vec.push_back(new cmd_opt);
   option_vec.push_back(new auto_analyze_opt);
 
+  // Feed provided script into interactive buffer
+  if (!s.script_file.empty())
+    {
+      ifstream ifs(s.script_file);
+      string line;
+      while (getline(ifs, line))
+        script_vec.push_back(line);
+    }
+  else if (!s.cmdline_script.empty())
+    script_vec.push_back(s.cmdline_script);
+  for (auto it = s.additional_scripts.begin();
+       it != s.additional_scripts.end(); ++it)
+    script_vec.push_back(*it);
+  s.script_file.clear();
+  s.cmdline_script.clear();
+  s.additional_scripts.clear();
+
   // FIXME: It might be better to wait to get the list of probes and
   // aliases until they are needed.
 
@@ -1734,7 +1948,7 @@ interactive_mode (systemtap_session &s, vector<remote*> targets)
       }
   }
 #endif
-
+  cout << "Example commands: 'sample', 'add', 'run', 'help'" << endl;
   while (1)
     {
       // PR19847: clear any pending interrupts from previous commands
@@ -1751,7 +1965,10 @@ interactive_mode (systemtap_session &s, vector<remote*> targets)
       else if (*line_tmp)
 	add_history(line_tmp);
       else
-	continue;
+	{
+	  cout << "Example commands: 'sample', 'add', 'run', 'help'" << endl;
+          continue;
+        }
 
       string line = string(line_tmp);
       free(line_tmp);
@@ -1767,7 +1984,7 @@ interactive_mode (systemtap_session &s, vector<remote*> targets)
 	  for (cmdopt_vector_iterator it = command_vec.begin();
 	       it != command_vec.end(); ++it)
 	    {
-	      if (tokens[0] == (*it)->name)
+	      if ((*it)->accept(tokens[0]))
 	        {
 		  input_handled = true;
 		  quit = (*it)->handler(s, tokens, line);
@@ -1782,7 +1999,7 @@ interactive_mode (systemtap_session &s, vector<remote*> targets)
       // If it isn't a command, complain.
       if (!input_handled)
 	clog << "Undefined command: \"" << tokens[0]
-	     << "\". Try \"help\"." << endl;
+	     << "\". Try \"help\" or \"?\"." << endl;
     }
   delete_match_map_items(&probe_map);
   return 0;
