@@ -1,5 +1,5 @@
 // systemtap translator/driver
-// Copyright (C) 2005-2014 Red Hat Inc.
+// Copyright (C) 2005-2016 Red Hat Inc.
 // Copyright (C) 2005 IBM Corp.
 // Copyright (C) 2006 Intel Corporation.
 //
@@ -25,6 +25,9 @@
 #include "remote.h"
 #include "tapsets.h"
 #include "setupdwfl.h"
+#ifdef HAVE_LIBREADLINE
+#include "interactive.h"
+#endif
 
 #if ENABLE_NLS
 #include <libintl.h>
@@ -34,6 +37,8 @@
 #include "stap-probe.h"
 
 #include <cstdlib>
+#include <thread>
+#include <algorithm>
 
 extern "C" {
 #include <glob.h>
@@ -46,6 +51,7 @@ extern "C" {
 #include <time.h>
 #include <unistd.h>
 #include <wordexp.h>
+#include <ftw.h>
 }
 
 using namespace std;
@@ -126,7 +132,11 @@ printscript(systemtap_session& s, ostream& o)
       // print probe name and variables if there
       for (map<string, set<derived_probe *> >::iterator it=probe_list.begin(); it!=probe_list.end(); ++it)
         {
-          o << it->first; // probe name or alias
+          // probe name or alias
+          if (s.dump_mode == systemtap_session::dump_matched_probes_vars && isatty(STDOUT_FILENO))
+            o << s.colorize(it->first, "source");
+          else
+            o << it->first;
 
           // Print the locals and arguments for -L mode only
           if (s.dump_mode == systemtap_session::dump_matched_probes_vars)
@@ -252,7 +262,7 @@ printscript(systemtap_session& s, ostream& o)
 int pending_interrupts;
 
 extern "C"
-void handle_interrupt (int sig)
+void handle_interrupt (int)
 {
   // This might be nice, but we don't know our current verbosity...
   // clog << _F("Received signal %d", sig) << endl << flush;
@@ -300,15 +310,13 @@ setup_signals (sighandler_t handler)
 }
 
 
-static void*
-sdt_benchmark_thread(void* p)
+static void
+sdt_benchmark_thread(unsigned long i)
 {
-  unsigned long i = *(unsigned long*)p;
   PROBE(stap, benchmark__thread__start);
   while (i--)
     PROBE1(stap, benchmark, i);
   PROBE(stap, benchmark__thread__end);
-  return NULL;
 }
 
 
@@ -329,13 +337,13 @@ run_sdt_benchmark(systemtap_session& s)
   gettimeofday (&tv_before, NULL);
 
   PROBE(stap, benchmark__start);
-
-  pthread_t pthreads[threads];
-  for (unsigned long i = 0; i < threads; ++i)
-    pthread_create(&pthreads[i], NULL, sdt_benchmark_thread, &loops);
-  for (unsigned long i = 0; i < threads; ++i)
-    pthread_join(pthreads[i], NULL);
-
+    {
+      vector<thread> handles;
+      for (unsigned long i = 0; i < threads; ++i)
+        handles.push_back(thread(sdt_benchmark_thread, loops));
+      for (unsigned long i = 0; i < threads; ++i)
+        handles[i].join();
+    }
   PROBE(stap, benchmark__end);
 
   times (& tms_after);
@@ -351,9 +359,49 @@ run_sdt_benchmark(systemtap_session& s)
   return EXIT_SUCCESS;
 }
 
+static set<string> files;
+static string path_dir;
+
+static int collect_stp(const char* fpath, const struct stat*,
+                       int typeflag, struct FTW* ftwbuf)
+{
+  if (typeflag == FTW_F)
+    {
+      const char* ext = strrchr(fpath, '.');
+      if (ext && (strcmp(".stp", ext) == 0))
+        files.insert(fpath);
+    }
+  else if (typeflag == FTW_D && ftwbuf->level > 0)
+    {
+      // Only recurse for PATH root directory
+      if (strncmp(path_dir.c_str(), fpath, path_dir.size()) != 0 ||
+          (fpath[path_dir.size()] != '/' && fpath[path_dir.size()] != '\0'))
+        return FTW_SKIP_SUBTREE;
+    }
+  return FTW_CONTINUE;
+}
+
+static int collect_stpm(const char* fpath, const struct stat*,
+                        int typeflag, struct FTW* ftwbuf)
+{
+  if (typeflag == FTW_F)
+    {
+      const char* ext = strrchr(fpath, '.');
+      if (ext && (strcmp(".stpm", ext) == 0))
+        files.insert(fpath);
+    }
+  else if (typeflag == FTW_D && ftwbuf->level > 0)
+    {
+      // Only recurse for PATH root directory
+      if (strncmp(path_dir.c_str(), fpath, path_dir.size()) != 0 ||
+          (fpath[path_dir.size()] != '/' && fpath[path_dir.size()] != '\0'))
+        return FTW_SKIP_SUBTREE;
+    }
+  return FTW_CONTINUE;
+}
 
 // Compilation passes 0 through 4
-static int
+int
 passes_0_4 (systemtap_session &s)
 {
   int rc = 0;
@@ -433,253 +481,277 @@ passes_0_4 (systemtap_session &s)
   // PASS 1a: PARSING LIBRARY SCRIPTS
   PROBE1(stap, pass1a__start, &s);
 
-  // We need to handle the library scripts first because this pass
-  // gathers information on .stpm files that might be needed to
-  // parse the user script.
-
-  // We need to first ascertain the status of the user script, though.
-  struct stat user_file_stat;
-  int user_file_stat_rc = -1;
-
-  if (s.script_file == "-")
+  // prep this array for tapset $n use too ... although we will reset once again for user scripts
+  s.used_args.resize(s.args.size(), false);
+  
+  if (! s.pass_1a_complete)
     {
-      user_file_stat_rc = fstat (STDIN_FILENO, & user_file_stat);
-    }
-  else if (s.script_file != "")
-    {
-      user_file_stat_rc = stat (s.script_file.c_str(), & user_file_stat);
-    }
-  // otherwise, rc is 0 for a command line script
+      // We need to handle the library scripts first because this pass
+      // gathers information on .stpm files that might be needed to
+      // parse the user script.
 
-  vector<string> version_suffixes;
-  if (s.runtime_mode == systemtap_session::kernel_runtime)
-    {
-      // Construct kernel-versioning search path
-      string kvr = s.kernel_release;
+      // We need to first ascertain the status of the user script, though.
+      struct stat user_file_stat;
+      int user_file_stat_rc = -1;
 
-      // add full kernel-version-release (2.6.NN-FOOBAR)
-      version_suffixes.push_back ("/" + kvr);
-
-      // add kernel version (2.6.NN)
-      if (kvr != s.kernel_base_release)
+      if (s.script_file == "-")
         {
-          kvr = s.kernel_base_release;
-          version_suffixes.push_back ("/" + kvr);
-        }
-
-      // add kernel family (2.6)
-      string::size_type dot1_index = kvr.find ('.');
-      string::size_type dot2_index = kvr.rfind ('.');
-      while (dot2_index > dot1_index && dot2_index != string::npos)
+	  user_file_stat_rc = fstat (STDIN_FILENO, & user_file_stat);
+	}
+      else if (s.script_file != "")
         {
-          kvr.erase(dot2_index);
-          version_suffixes.push_back ("/" + kvr);
-          dot2_index = kvr.rfind ('.');
-        }
-    }
+	  user_file_stat_rc = stat (s.script_file.c_str(), & user_file_stat);
+	}
+      // otherwise, rc is 0 for a command line script
 
-  // add empty string as last element
-  version_suffixes.push_back ("");
-
-  // Add arch variants of every path, just before each
-  const string& arch = s.architecture;
-  for (unsigned i=0; i<version_suffixes.size(); i+=2)
-    version_suffixes.insert(version_suffixes.begin() + i,
-                            version_suffixes[i] + "/" + arch);
-
-  // Add runtime variants of every path, before everything else
-  string runtime_prefix;
-  if (s.runtime_mode == systemtap_session::kernel_runtime)
-    runtime_prefix = "/linux";
-  else if (s.runtime_mode == systemtap_session::dyninst_runtime)
-    runtime_prefix = "/dyninst";
-  if (!runtime_prefix.empty())
-    for (unsigned i=0; i<version_suffixes.size(); i+=2)
-      version_suffixes.insert(version_suffixes.begin() + i/2,
-                              runtime_prefix + version_suffixes[i]);
-
-  // First, parse .stpm files on the include path. We need to have the
-  // resulting macro definitions available for parsing library files,
-  // but since .stpm files can consist only of '@define' constructs,
-  // we can parse each one without reference to the others.
-  set<pair<dev_t, ino_t> > seen_library_macro_files;
-  set<string> seen_library_macro_files_names;
-
-  for (unsigned i=0; i<s.include_path.size(); i++)
-    {
-      // now iterate upon it
-      for (unsigned k=0; k<version_suffixes.size(); k++)
+      vector<string> version_suffixes;
+      if (s.runtime_mode == systemtap_session::kernel_runtime)
         {
-          glob_t globbuf;
-          string dir = s.include_path[i] + version_suffixes[k] + "/*.stpm";
-          int r = glob(dir.c_str (), 0, NULL, & globbuf);
-          if (r == GLOB_NOSPACE || r == GLOB_ABORTED)
-            rc ++;
-          // GLOB_NOMATCH is acceptable
+	  // Construct kernel-versioning search path
+	  string kvr = s.kernel_release;
 
-          unsigned prev_s_library_files = s.library_files.size();
+	  // add full kernel-version-release (2.6.NN-FOOBAR)
+	  version_suffixes.push_back ("/" + kvr);
 
-          for (unsigned j=0; j<globbuf.gl_pathc; j++)
-            {
-              assert_no_interrupts();
+	  // add kernel version (2.6.NN)
+	  if (kvr != s.kernel_base_release)
+	    {
+	      kvr = s.kernel_base_release;
+	      version_suffixes.push_back ("/" + kvr);
+	    }
 
-              struct stat tapset_file_stat;
-              int stat_rc = stat (globbuf.gl_pathv[j], & tapset_file_stat);
-              if (stat_rc == 0 && user_file_stat_rc == 0 &&
-                  user_file_stat.st_dev == tapset_file_stat.st_dev &&
-                  user_file_stat.st_ino == tapset_file_stat.st_ino)
-                {
-                  cerr
-                  << _F("usage error: macro tapset file '%s' cannot be run directly as a session script.",
-                        globbuf.gl_pathv[j]) << endl;
-                  rc ++;
-                }
+	  // add kernel family (2.6)
+	  string::size_type dot1_index = kvr.find ('.');
+	  string::size_type dot2_index = kvr.rfind ('.');
+	  while (dot2_index > dot1_index && dot2_index != string::npos)
+	    {
+	      kvr.erase(dot2_index);
+	      version_suffixes.push_back ("/" + kvr);
+	      dot2_index = kvr.rfind ('.');
+	    }
+	}
 
-              // PR11949: duplicate-eliminate tapset files
-              if (stat_rc == 0)
-                {
-                  pair<dev_t,ino_t> here = make_pair(tapset_file_stat.st_dev,
-                                                     tapset_file_stat.st_ino);
-                  if (seen_library_macro_files.find(here) != seen_library_macro_files.end()) {
-                    if (s.verbose>2)
-                      clog << _F("Skipping tapset \"%s\", duplicate inode.", globbuf.gl_pathv[j]) << endl;
-                    continue; 
-                  }
-                  seen_library_macro_files.insert (here);
-                }
+      // add empty string as last element
+      version_suffixes.push_back ("");
 
-              // PR12443: duplicate-eliminate harder
-              string full_path = globbuf.gl_pathv[j];
-              string tapset_base = s.include_path[i]; // not dir; it has arch suffixes too
-              if (full_path.size() > tapset_base.size()) {
-                string tail_part = full_path.substr(tapset_base.size());
-                if (seen_library_macro_files_names.find (tail_part) != seen_library_macro_files_names.end()) {
-                  if (s.verbose>2)
-                      clog << _F("Skipping tapset \"%s\", duplicate name.", globbuf.gl_pathv[j]) << endl;
-                  continue;
-                }
-                seen_library_macro_files_names.insert (tail_part);
-              }
+      // Add arch variants of every path, just before each
+      const string& arch = s.architecture;
+      for (unsigned i=0; i<version_suffixes.size(); i+=2)
+	version_suffixes.insert(version_suffixes.begin() + i,
+				version_suffixes[i] + "/" + arch);
 
-              if (s.verbose>2)
-                clog << _F("Processing tapset \"%s\"", globbuf.gl_pathv[j]) << endl;
+      // Add runtime variants of every path, before everything else
+      string runtime_prefix;
+      if (s.runtime_mode == systemtap_session::kernel_runtime)
+	runtime_prefix = "/linux";
+      else if (s.runtime_mode == systemtap_session::dyninst_runtime)
+	runtime_prefix = "/dyninst";
+      if (!runtime_prefix.empty())
+	for (unsigned i=0; i<version_suffixes.size(); i+=2)
+	    version_suffixes.insert(version_suffixes.begin() + i/2,
+				    runtime_prefix + version_suffixes[i]);
 
-              stapfile* f = parse_library_macros (s, globbuf.gl_pathv[j]);
-              if (f == 0)
-                s.print_warning(_F("macro tapset \"%s\" has errors, and will be skipped.", string(globbuf.gl_pathv[j]).c_str()));
-              else
-                s.library_files.push_back (f);
-            }
+      // First, parse .stpm files on the include path. We need to have the
+      // resulting macro definitions available for parsing library files,
+      // but since .stpm files can consist only of '@define' constructs,
+      // we can parse each one without reference to the others.
+      set<pair<dev_t, ino_t> > seen_library_macro_files;
+      set<string> seen_library_macro_files_names;
 
-          unsigned next_s_library_files = s.library_files.size();
-          if (s.verbose>1 && globbuf.gl_pathc > 0)
-            //TRANSLATORS: Searching through directories, 'processed' means 'examined so far'
-            clog << _F("Searched for library macro files: \"%s\", found: %zu, processed: %u",
-                       dir.c_str(), globbuf.gl_pathc,
-                       (next_s_library_files-prev_s_library_files)) << endl;
-
-          globfree (&globbuf);
-        }
-    }
-
-  // Next, gather and parse the library files.
-  set<pair<dev_t, ino_t> > seen_library_files;
-  set<string> seen_library_files_names;
-
-  for (unsigned i=0; i<s.include_path.size(); i++)
-    {
-      unsigned tapset_flags = pf_guru | pf_squash_errors;
-
-      // The first path is special, as it's the builtin tapset.
-      // Allow all features no matter what s.compatible says.
-      if (i == 0)
-        tapset_flags |= pf_no_compatible;
-
-      // now iterate upon it
-      for (unsigned k=0; k<version_suffixes.size(); k++)
+      for (unsigned i=0; i<s.include_path.size(); i++)
         {
-          glob_t globbuf;
-          string dir = s.include_path[i] + version_suffixes[k] + "/*.stp";
-          int r = glob(dir.c_str (), 0, NULL, & globbuf);
-          if (r == GLOB_NOSPACE || r == GLOB_ABORTED)
-	    rc ++;
-	  // GLOB_NOMATCH is acceptable
+	  // now iterate upon it
+	  for (unsigned k=0; k<version_suffixes.size(); k++)
+	    {
+              int flags = FTW_ACTIONRETVAL;
+	      string dir = s.include_path[i] + version_suffixes[k];
+              files.clear();
+              // we need to set this for the nftw() callback
+              path_dir = s.include_path[i] + "/PATH";
+              (void) nftw(dir.c_str(), collect_stpm, 1, flags);
 
-          unsigned prev_s_library_files = s.library_files.size();
+	      unsigned prev_s_library_files = s.library_files.size();
 
-          for (unsigned j=0; j<globbuf.gl_pathc; j++)
-            {
-              assert_no_interrupts();
+	      for (auto it = files.begin(); it != files.end(); ++it)
+	        {
+		  assert_no_interrupts();
 
-              struct stat tapset_file_stat;
-              int stat_rc = stat (globbuf.gl_pathv[j], & tapset_file_stat);
-              if (stat_rc == 0 && user_file_stat_rc == 0 &&
-                  user_file_stat.st_dev == tapset_file_stat.st_dev &&
-                  user_file_stat.st_ino == tapset_file_stat.st_ino)
-                {
-                  cerr 
-                  << _F("usage error: tapset file '%s' cannot be run directly as a session script.",
-                        globbuf.gl_pathv[j]) << endl;
-                  rc ++;
-                }
+		  struct stat tapset_file_stat;
+		  int stat_rc = stat (it->c_str(), & tapset_file_stat);
+		  if (stat_rc == 0 && user_file_stat_rc == 0 &&
+		      user_file_stat.st_dev == tapset_file_stat.st_dev &&
+		      user_file_stat.st_ino == tapset_file_stat.st_ino)
+		    {
+		      cerr
+			  << _F("usage error: macro tapset file '%s' cannot be run directly as a session script.",
+				it->c_str()) << endl;
+		      rc ++;
+		    }
 
-              // PR11949: duplicate-eliminate tapset files
-              if (stat_rc == 0)
-                {
-                  pair<dev_t,ino_t> here = make_pair(tapset_file_stat.st_dev,
-                                                     tapset_file_stat.st_ino);
-                  if (seen_library_files.find(here) != seen_library_files.end()) {
-                    if (s.verbose>2)
-                      clog << _F("Skipping tapset \"%s\", duplicate inode.", globbuf.gl_pathv[j]) << endl;
-                    continue; 
-                  }
-                  seen_library_files.insert (here);
-                }
+		  // PR11949: duplicate-eliminate tapset files
+		  if (stat_rc == 0)
+		    {
+		      pair<dev_t,ino_t> here = make_pair(tapset_file_stat.st_dev,
+							 tapset_file_stat.st_ino);
+		      if (seen_library_macro_files.find(here) != seen_library_macro_files.end())
+		        {
+			  if (s.verbose>2)
+			    clog << _F("Skipping tapset \"%s\", duplicate inode.", it->c_str()) << endl;
+			  continue; 
+			}
+		      seen_library_macro_files.insert (here);
+		    }
 
-              // PR12443: duplicate-eliminate harder
-              string full_path = globbuf.gl_pathv[j];
-              string tapset_base = s.include_path[i]; // not dir; it has arch suffixes too
-              if (full_path.size() > tapset_base.size()) {
-                string tail_part = full_path.substr(tapset_base.size());
-                if (seen_library_files_names.find (tail_part) != seen_library_files_names.end()) {
-                  if (s.verbose>2)
-                      clog << _F("Skipping tapset \"%s\", duplicate name.", globbuf.gl_pathv[j]) << endl;
-                  continue;
-                }
-                seen_library_files_names.insert (tail_part);
-              }
+		  // PR12443: duplicate-eliminate harder
+		  string full_path = *it;
+		  string tapset_base = s.include_path[i]; // not dir; it has arch suffixes too
+		  if (full_path.size() > tapset_base.size())
+		    {
+		      string tail_part = full_path.substr(tapset_base.size());
+		      if (seen_library_macro_files_names.find (tail_part) != seen_library_macro_files_names.end())
+		        {
+			  if (s.verbose>2)
+			    clog << _F("Skipping tapset \"%s\", duplicate name.", it->c_str()) << endl;
+			  continue;
+			}
+		      seen_library_macro_files_names.insert (tail_part);
+		    }
 
-              if (s.verbose>2)
-                clog << _F("Processing tapset \"%s\"", globbuf.gl_pathv[j]) << endl;
+		  if (s.verbose>2)
+		    clog << _F("Processing tapset \"%s\"", it->c_str()) << endl;
 
-              // NB: we don't need to restrict privilege only for /usr/share/systemtap, i.e., 
-              // excluding user-specified $XDG_DATA_DIRS.  That's because stapdev gets
-              // root-equivalent privileges anyway; stapsys and stapusr use a remote compilation
-              // with a trusted environment, where client-side $XDG_DATA_DIRS are not passed.
+		  stapfile* f = parse_library_macros (s, *it);
+		  if (f == 0)
+		    s.print_warning(_F("macro tapset \"%s\" has errors, and will be skipped.", it->c_str()));
+		  else
+		    s.library_files.push_back (f);
+		}
 
-              stapfile* f = parse (s, globbuf.gl_pathv[j], tapset_flags);
-              if (f == 0)
-                s.print_warning(_F("tapset \"%s\" has errors, and will be skipped", string(globbuf.gl_pathv[j]).c_str()));
-              else
-                s.library_files.push_back (f);
-            }
+	      unsigned next_s_library_files = s.library_files.size();
+	      if (s.verbose>1 && !files.empty())
+		  //TRANSLATORS: Searching through directories, 'processed' means 'examined so far'
+		clog << _F("Searched for library macro files: \"%s\", found: %zu, processed: %u",
+			   dir.c_str(), files.size(),
+			   (next_s_library_files-prev_s_library_files)) << endl;
+	    }
+	}
 
-          unsigned next_s_library_files = s.library_files.size();
-          if (s.verbose>1 && globbuf.gl_pathc > 0)
-            //TRANSLATORS: Searching through directories, 'processed' means 'examined so far'
-            clog << _F("Searched: \"%s\", found: %zu, processed: %u",
-                       dir.c_str(), globbuf.gl_pathc,
-                       (next_s_library_files-prev_s_library_files)) << endl;
+      // Next, gather and parse the library files.
+      set<pair<dev_t, ino_t> > seen_library_files;
+      set<string> seen_library_files_names;
 
-          globfree (& globbuf);
-        }
-    }
-  if (s.num_errors())
-    rc ++;
+      for (unsigned i=0; i<s.include_path.size(); i++)
+        {
+	  // now iterate upon it
+	  for (unsigned k=0; k<version_suffixes.size(); k++)
+	    {
+              int flags = FTW_ACTIONRETVAL;
+	      string dir = s.include_path[i] + version_suffixes[k];
+              files.clear();
+              // we need to set this for the nftw() callback
+              path_dir = s.include_path[i] + "/PATH";
+              (void) nftw(dir.c_str(), collect_stp, 1, flags);
+
+	      unsigned prev_s_library_files = s.library_files.size();
+
+              for (auto it = files.begin(); it != files.end(); ++it)
+	        {
+                  unsigned tapset_flags = pf_guru | pf_squash_errors;
+
+                  // The first path is special, as it's the builtin tapset.
+                  // Allow all features no matter what s.compatible says.
+                  if (i == 0)
+                    tapset_flags |= pf_no_compatible;
+
+                  if (it->find("/PATH/") != string::npos)
+                    tapset_flags |= pf_auto_path;
+
+                  assert_no_interrupts();
+
+		  struct stat tapset_file_stat;
+		  int stat_rc = stat (it->c_str(), & tapset_file_stat);
+		  if (stat_rc == 0 && user_file_stat_rc == 0 &&
+		      user_file_stat.st_dev == tapset_file_stat.st_dev &&
+		      user_file_stat.st_ino == tapset_file_stat.st_ino)
+		    {
+		      cerr 
+			  << _F("usage error: tapset file '%s' cannot be run directly as a session script.",
+				it->c_str()) << endl;
+		      rc ++;
+		    }
+
+		  // PR11949: duplicate-eliminate tapset files
+		  if (stat_rc == 0)
+		    {
+		      pair<dev_t,ino_t> here = make_pair(tapset_file_stat.st_dev,
+							 tapset_file_stat.st_ino);
+		      if (seen_library_files.find(here) != seen_library_files.end())
+		        {
+			  if (s.verbose>2)
+			    clog << _F("Skipping tapset \"%s\", duplicate inode.", it->c_str()) << endl;
+			  continue; 
+			}
+		      seen_library_files.insert (here);
+		    }
+
+		  // PR12443: duplicate-eliminate harder
+		  string full_path = *it;
+		  string tapset_base = s.include_path[i]; // not dir; it has arch suffixes too
+		  if (full_path.size() > tapset_base.size())
+		    {
+		      string tail_part = full_path.substr(tapset_base.size());
+		      if (seen_library_files_names.find (tail_part) != seen_library_files_names.end())
+		        {
+			  if (s.verbose>2)
+			    clog << _F("Skipping tapset \"%s\", duplicate name.", it->c_str()) << endl;
+			  continue;
+			}
+		      seen_library_files_names.insert (tail_part);
+		    }
+
+		  if (s.verbose>2)
+		    clog << _F("Processing tapset \"%s\"", it->c_str()) << endl;
+
+		  // NB: we don't need to restrict privilege only for
+		  // /usr/share/systemtap, i.e., excluding
+		  // user-specified $XDG_DATA_DIRS.  That's because
+		  // stapdev gets root-equivalent privileges anyway;
+		  // stapsys and stapusr use a remote compilation with
+		  // a trusted environment, where client-side
+		  // $XDG_DATA_DIRS are not passed.
+
+		  stapfile* f = parse (s, *it, tapset_flags);
+		  if (f == 0)
+		    s.print_warning(_F("tapset \"%s\" has errors, and will be skipped", it->c_str()));
+		  else
+		    s.library_files.push_back (f);
+		}
+
+	      unsigned next_s_library_files = s.library_files.size();
+	      if (s.verbose>1 && !files.empty())
+		  //TRANSLATORS: Searching through directories, 'processed' means 'examined so far'
+		clog << _F("Searched: \"%s\", found: %zu, processed: %u",
+			   dir.c_str(), files.size(),
+			   (next_s_library_files-prev_s_library_files)) << endl;
+	    }
+	}
+
+      if (s.num_errors())
+	rc ++;
+
+      // Now that we've made it through pass 1a, remember this so we
+      // don't have to do this again in interactive mode. This doesn't
+      // effect non-interactive mode.
+      s.pass_1a_complete = true;
+  }
 
   // PASS 1b: PARSING USER SCRIPT
   PROBE1(stap, pass1b__start, &s);
 
+  // reset for user scripts -- it's their use of $* we care about
+  // except that tapsets like argv.stp can consume $parms
+  fill(s.used_args.begin(), s.used_args.end(), false);
+
   // Only try to parse a user script if the user provided one, or if we have to
   // make one (as is the case for listing mode). Otherwise, s.user_script
   // remains NULL.
@@ -689,6 +761,7 @@ passes_0_4 (systemtap_session &s)
       s.dump_mode == systemtap_session::dump_matched_probes_vars)
     {
       unsigned user_flags = s.guru_mode ? pf_guru : 0;
+      user_flags |= pf_user_file;
       if (s.script_file == "-")
         {
           s.user_files.push_back (parse (s, "<input>", cin, user_flags));
@@ -730,6 +803,12 @@ passes_0_4 (systemtap_session &s)
             }
         }
     }
+  else if (s.cmdline_script.empty() &&
+           s.dump_mode == systemtap_session::dump_none) // -e ''
+    {
+      cerr << _("Input file '<input>' is empty.") << endl;
+      rc++;
+    }
 
   // Dump a list of probe aliases picked up, if requested
   if (s.dump_mode == systemtap_session::dump_probe_aliases)
@@ -780,7 +859,7 @@ passes_0_4 (systemtap_session &s)
   struct timeval tv_after;
   gettimeofday (&tv_after, NULL);
 
-#define TIMESPRINT "in " << \
+#define TIMESPRINT _("in ") << \
            (tms_after.tms_cutime + tms_after.tms_utime \
             - tms_before.tms_cutime - tms_before.tms_utime) * 1000 / (_sc_clk_tck) << "usr/" \
         << (tms_after.tms_cstime + tms_after.tms_stime \
@@ -792,9 +871,9 @@ passes_0_4 (systemtap_session &s)
   if (s.verbose)
     {
       // XXX also include a count of helper macro files loaded (.stpm)?
-      clog << "Pass 1: parsed user script and "
-           << s.library_files.size()
-           << " library script(s) "
+      int n = int(s.library_files.size());
+      clog << _("Pass 1: parsed user script and ")
+           << _NF("%d library script ", "%d library scripts ", n, n)
            << getmemusage()
            << TIMESPRINT
            << endl;
@@ -831,7 +910,9 @@ passes_0_4 (systemtap_session &s)
           functiondecl& curfunc = *func->second;
           if (curfunc.synthetic)
             continue;
-          if (!s.verbose && startswith(curfunc.name, "_"))
+          if (!startswith(curfunc.name, "__global_"))
+            continue;
+          if (!s.verbose && startswith(curfunc.name, "__global__"))
             continue;
           curfunc.printsigtags(cout, s.verbose>0 /* all_tags */ );
           cout << endl;
@@ -840,21 +921,47 @@ passes_0_4 (systemtap_session &s)
   // Dump the whole script if requested, or if we stop at 2
   else if (s.dump_mode == systemtap_session::dump_matched_probes ||
            s.dump_mode == systemtap_session::dump_matched_probes_vars ||
-           (rc == 0 && s.last_pass == 2))
+           (rc == 0 && s.last_pass == 2) ||
+           (rc != 0 && s.verbose > 2))
     printscript(s, cout);
 
   times (& tms_after);
   gettimeofday (&tv_after, NULL);
 
-  if (s.verbose) clog << "Pass 2: analyzed script: "
-                      << s.probes.size() << " probe(s), "
-                      << s.functions.size() << " function(s), "
-                      << s.embeds.size() << " embed(s), "
-                      << s.globals.size() << " global(s) "
-                      << getmemusage()
-                      << TIMESPRINT
-                      << endl;
+  if (s.verbose) {
+    int np = s.probes.size();
+    int nf = s.functions.size();
+    int ne = s.embeds.size();
+    int ng = s.globals.size();
+    clog << _("Pass 2: analyzed script: ")
+         << _NF("%d probe, ", "%d probes, ", np, np)
+         << _NF("%d function, ", "%d functions, ", nf, nf)
+         << _NF("%d embed, ", "%d embeds, ", ne, ne)
+         << _NF("%d global ", "%d globals ", ng, ng)
+         << getmemusage()
+         << TIMESPRINT
+         << endl;
+  }
 
+  missing_rpm_list_print(s, "-debuginfo");
+
+  // Check for unused command line parameters.  But - if the argv
+  // tapset was selected for inclusion, then the user-script need not
+  // use $* directly, so we want to suppress the warning in this case.
+  // This is hacky, but we don't have a formal way of tracking tokens
+  // that came from command line arguments so as to do set-subtraction
+  // at this point.
+  //
+  bool argc_found=false, argv_found=false;
+  for (unsigned i = 0; i<s.globals.size(); i++) {
+    if (s.globals[i]->unmangled_name == "argc") argc_found = true;
+    if (s.globals[i]->unmangled_name == "argv") argv_found = true;
+  }
+  if (!argc_found && !argv_found)
+    for (unsigned i = 0; i<s.used_args.size(); i++)
+      if (! s.used_args[i])
+        s.print_warning (_F("unused command line option $%u/@%u", i+1, i+1));
+  
   if (rc && !s.dump_mode && !s.try_server ())
     cerr << _("Pass 2: analysis failed.  [man error::pass2]") << endl;
 
@@ -923,7 +1030,7 @@ passes_0_4 (systemtap_session &s)
   gettimeofday (&tv_after, NULL);
 
   if (s.verbose) 
-    clog << "Pass 3: translated to C into \""
+    clog << _("Pass 3: translated to C into \"")
          << s.translated_source
          << "\" "
          << getmemusage()
@@ -959,7 +1066,7 @@ passes_0_4 (systemtap_session &s)
   times (& tms_after);
   gettimeofday (&tv_after, NULL);
 
-  if (s.verbose) clog << "Pass 4: compiled C into \""
+  if (s.verbose) clog << _("Pass 4: compiled C into \"")
                       << s.module_filename()
                       << "\" "
                       << TIMESPRINT
@@ -1003,7 +1110,7 @@ passes_0_4 (systemtap_session &s)
   return rc;
 }
 
-static int
+int
 pass_5 (systemtap_session &s, vector<remote*> targets)
 {
   // PASS 5: RUN
@@ -1023,7 +1130,7 @@ pass_5 (systemtap_session &s, vector<remote*> targets)
   unsigned _sc_clk_tck = sysconf (_SC_CLK_TCK);
   struct timeval tv_after;
   gettimeofday (&tv_after, NULL);
-  if (s.verbose) clog << "Pass 5: run completed "
+  if (s.verbose) clog << _("Pass 5: run completed ")
                       << TIMESPRINT
                       << endl;
 
@@ -1115,6 +1222,8 @@ main (int argc, char * const argv [])
         linecount ++;
         if (rc) break;
       }
+    rcf.close();
+
     int extended_argc = words.we_wordc + argc;
     char **extended_argv = (char**) calloc (extended_argc + 1, sizeof(char*));
     if (rc || !extended_argv)
@@ -1134,6 +1243,9 @@ main (int argc, char * const argv [])
     if (rc != 0)
       return rc;
 
+    // Create the temp dir.
+    s.create_tmp_dir();
+
     if (words.we_wordc > 0 && s.verbose > 1)
       clog << _F("Extra options in %s: %d\n", rc_file.c_str(), (int)words.we_wordc);
 
@@ -1180,45 +1292,60 @@ main (int argc, char * const argv [])
     set<systemtap_session*> sessions;
     for (unsigned i = 0; i < targets.size(); ++i)
       sessions.insert(targets[i]->get_session());
-    for (set<systemtap_session*>::iterator it = sessions.begin();
-         rc == 0 && !pending_interrupts && it != sessions.end(); ++it)
+
+    // FIXME: For now, only attempt local interactive use.
+    if (s.interactive_mode && fake_remote)
       {
-        systemtap_session& ss = **it;
-        if (ss.verbose > 1)
-          clog << _F("Session arch: %s release: %s",
-                     ss.architecture.c_str(), ss.kernel_release.c_str()) << endl;
+#ifdef HAVE_LIBREADLINE
+	rc = interactive_mode (s, targets);
+#endif
+      }
+    else
+      {
+	for (set<systemtap_session*>::iterator it = sessions.begin();
+	     rc == 0 && !pending_interrupts && it != sessions.end(); ++it)
+	  {
+	    systemtap_session& ss = **it;
+            if (ss.verbose > 1)
+	      clog << _F("Session arch: %s release: %s",
+			 ss.architecture.c_str(), ss.kernel_release.c_str())
+		   << endl;
 
 #if HAVE_NSS
-        // If requested, query server status. This is independent of other tasks.
-        query_server_status (ss);
+	    // If requested, query server status. This is independent
+	    // of other tasks.
+	    query_server_status (ss);
 
-        // If requested, manage trust of servers. This is independent of other tasks.
-        manage_server_trust (ss);
+	    // If requested, manage trust of servers. This is
+	    // independent of other tasks.
+	    manage_server_trust (ss);
 #endif
 
-        // Run the passes only if a script has been specified or if we're
-        // dumping something. The requirement for a script has already been
-        // checked in systemtap_session::check_options.
-        if (ss.have_script || ss.dump_mode)
-          {
-            // Run passes 0-4 for each unique session,
-            // either locally or using a compile-server.
-            ss.init_try_server ();
-            if ((rc = passes_0_4 (ss)))
-              {
-                // Compilation failed.
-                // Try again using a server if appropriate.
-                if (ss.try_server ())
-                  rc = passes_0_4_again_with_server (ss);
-              }
-	    if (rc || s.perpass_verbose[0] >= 1)
-	      s.explain_auto_options ();
-          }
-      }
+	    // Run the passes only if a script has been specified or
+	    // if we're dumping something. The requirement for a
+	    // script has already been checked in
+	    // systemtap_session::check_options.
+	    if (ss.have_script || ss.dump_mode)
+	      {
+		// Run passes 0-4 for each unique session, either
+		// locally or using a compile-server.
+		ss.init_try_server ();
+		if ((rc = passes_0_4 (ss)))
+		  {
+		    // Compilation failed.
+		    // Try again using a server if appropriate.
+		    if (ss.try_server ())
+		      rc = passes_0_4_again_with_server (ss);
+		  }
+		if (rc || s.perpass_verbose[0] >= 1)
+		  s.explain_auto_options ();
+	      }
+	  }
 
-    // Run pass 5, if requested
-    if (rc == 0 && s.have_script && s.last_pass >= 5 && ! pending_interrupts)
-      rc = pass_5 (s, targets);
+	// Run pass 5, if requested
+	if (rc == 0 && s.have_script && s.last_pass >= 5 && ! pending_interrupts)
+	  rc = pass_5 (s, targets);
+      }
 
     // Pass 6. Cleanup
     for (unsigned i = 0; i < targets.size(); ++i)
