@@ -11,13 +11,6 @@
 #ifndef _LINUX_STAT_RUNTIME_H_
 #define _LINUX_STAT_RUNTIME_H_
 
-/* for the paranoid. */
-#ifdef NEED_STAT_LOCKS
-#define STAT_LOCK(sd)		spin_lock(&sd->lock)
-#define STAT_UNLOCK(sd)		spin_unlock(&sd->lock)
-#define STAT_GET_CPU()		get_cpu()
-#define STAT_PUT_CPU()		put_cpu()
-#else
 #define STAT_LOCK(sd)		do {} while (0)
 #define STAT_UNLOCK(sd)		do {} while (0)
 /* get/put_cpu wrappers.  Unnecessary if caller is already atomic. */
@@ -27,7 +20,6 @@
 #define STAT_GET_CPU()		smp_processor_id()
 #endif
 #define STAT_PUT_CPU()		do {} while (0)
-#endif
 
 /** Stat struct. Maps do not need this */
 typedef struct _Stat {
@@ -80,19 +72,4 @@ static void _stp_stat_free(Stat st)
 #define _stp_stat_get_agg(stat) ((stat)->agg)
 #define _stp_stat_per_cpu_ptr(stat, cpu) per_cpu_ptr((stat)->sd, (cpu))
 
-static int _stp_stat_initialize_locks(Stat st)
-{
-#ifdef NEED_STAT_LOCKS
-	int i;
-	for_each_possible_cpu(i) {
-		stat_data *sdp = _stp_stat_per_cpu_ptr(st, i);
-		spin_lock_init(&sdp->lock);
-	}
-	spin_lock_init(&st->agg->lock);
-#endif
-	return 0;
-}
-
-#define _stp_stat_destroy_locks(st)	do {} while (0)
-
 #endif /* _LINUX_STAT_RUNTIME_H_ */
