@@ -1,6 +1,6 @@
 /* -*- linux-c -*- 
  * map API generator
- * Copyright (C) 2005 Red Hat Inc.
+ * Copyright (C) 2005-2016 Red Hat Inc.
  *
  * This file is part of systemtap, and is free software.  You can
  * redistribute it and/or modify it under the terms of the GNU General
@@ -48,8 +48,8 @@
 #define VALN s
 #define VALSTOR char value[MAP_STRING_LENGTH]
 #define MAP_GET_VAL(node) ((node)->value)
-#define MAP_SET_VAL(map,node,val,add) _new_map_set_str(map,MAP_GET_VAL(node),val,add)
-#define MAP_COPY_VAL(map,node,val,add) MAP_SET_VAL(map,node,val,add)
+#define MAP_SET_VAL(map,node,val,add,s1,s2,s3,s4,s5) _new_map_set_str(map,MAP_GET_VAL(node),val,add)
+#define MAP_COPY_VAL(map,node,val,add) MAP_SET_VAL(map,node,val,add,0,0,0,0,0)
 #define NULLRET ""
 #elif VALUE_TYPE == INT64
 #define VALTYPE int64_t
@@ -58,8 +58,8 @@
 #define VALN i
 #define VALSTOR int64_t value
 #define MAP_GET_VAL(node) ((node)->value)
-#define MAP_SET_VAL(map,node,val,add) _new_map_set_int64(map,&MAP_GET_VAL(node),val,add)
-#define MAP_COPY_VAL(map,node,val,add) MAP_SET_VAL(map,node,val,add)
+#define MAP_SET_VAL(map,node,val,add,s1,s2,s3,s4,s5) _new_map_set_int64(map,&MAP_GET_VAL(node),val,add)
+#define MAP_COPY_VAL(map,node,val,add) MAP_SET_VAL(map,node,val,add,0,0,0,0,0)
 #define NULLRET (int64_t)0
 #elif VALUE_TYPE == STAT
 #define VALTYPE stat_data*
@@ -68,7 +68,7 @@
 #define VALN x
 #define VALSTOR stat_data value
 #define MAP_GET_VAL(node) (&(node)->value)
-#define MAP_SET_VAL(map,node,val,add) _new_map_set_stat(map,MAP_GET_VAL(node),val,add)
+#define MAP_SET_VAL(map,node,val,add,s1,s2,s3,s4,s5) _new_map_set_stat(map,MAP_GET_VAL(node),val,add,s1,s2,s3,s4,s5)
 #define MAP_COPY_VAL(map,node,val,add) _new_map_copy_stat(map,MAP_GET_VAL(node),val,add)
 #define NULLRET (stat_data*)0
 #else
@@ -76,6 +76,50 @@
 #endif /* VALUE_TYPE */
 
 
+/* murmurhash3 body, for use in KEYSYM(hash)
+   Extracted from
+   https://github.com/aappleby/smhasher/tree/master/src
+   -----------------------------------------------------------------------------
+   MurmurHash3 was written by Austin Appleby, and is placed in the public
+   domain. The author hereby disclaims copyright to this source code.
+   -----------------------------------------------------------------------------
+*/
+#define ROTL32(x,r) (((uint32_t)x << r) | ((uint32_t)x >> (32 - r)))
+
+#define MURMUR_INT64(v) do {                   \
+                uint32_t k1;                    \
+                k1 = (v & 0xFFFFFFFF);                  \
+                k1 *= c1; k1 = ROTL32(k1,15); k1 *= c2;             \
+                h1 ^= k1; h1 = ROTL32(h1,13); h1 = h1*5+0xe6546b64; \
+                k1 = ((v >> 32) & 0xFFFFFFFF);                      \
+                k1 *= c1; k1 = ROTL32(k1,15); k1 *= c2;                 \
+                h1 ^= k1; h1 = ROTL32(h1,13); h1 = h1*5+0xe6546b64;     \
+                len += 8;                                               \
+        } while(0)
+#define MURMUR_STRING(v) do { \
+                uint32_t mylen = strlen(v); \
+                int nblocks = mylen / 4; \
+                const uint32_t * blocks = (const uint32_t *)(v + nblocks*4); \
+                const uint8_t * tail; \
+                uint32_t k1; \
+                int i; \
+                for(i = -nblocks; i; i++)  { \
+                        uint32_t k1 = blocks[i]; \
+                        k1 *= c1; k1 = ROTL32(k1,15); k1 *= c2; \
+                        h1 ^= k1; h1 = ROTL32(h1,13); h1 = h1*5+0xe6546b64; \
+                } \
+                tail = (const uint8_t*)(v + nblocks*4); \
+                k1 = 0; \
+                switch(mylen & 3) {                \
+                case 3: k1 ^= tail[2] << 16; \
+                case 2: k1 ^= tail[1] << 8; \
+                case 1: k1 ^= tail[0]; \
+                        k1 *= c1; k1 = ROTL32(k1,15); k1 *= c2; h1 ^= k1; \
+                } \
+                len += mylen; \
+        } while (0)
+
+        
 #if defined (KEY1_TYPE)
 #define KEY_ARITY 1
 #if KEY1_TYPE == STRING
@@ -84,15 +128,27 @@
 #define KEY1N s
 #define KEY1STOR char key1[MAP_STRING_LENGTH]
 #define KEY1CPY(m) str_copy(m->key1, key1)
+#define KEY1_HASH MURMUR_STRING(key1)
 #else
 #define KEY1TYPE int64_t
 #define KEY1NAME int64
 #define KEY1N i
 #define KEY1STOR int64_t key1
 #define KEY1CPY(m) m->key1=key1
+
+/* Instead of ...
+   #define KEY1_HASH MURMUR_INT64(key1)
+
+   We could do the quick & dirty kernel hash_64 for the first integer index.
+   This would be faster, and may give reasonable dispersion with the
+   retained final 32-bit mix.
+   #define KEY1_HASH h1 ^= hash_64(key1,32)
+
+   ... but hashing throughput is only slightly faster
+*/
+#define KEY1_HASH MURMUR_INT64(key1)
 #endif
 #define KEY1_EQ_P JOIN(KEY1NAME,eq_p)
-#define KEY1_HASH JOIN(KEY1NAME,hash)
 #endif /* defined(KEY1_TYPE) */
 
 #if defined (KEY2_TYPE)
@@ -104,15 +160,16 @@
 #define KEY2N s
 #define KEY2STOR char key2[MAP_STRING_LENGTH]
 #define KEY2CPY(m) str_copy(m->key2, key2)
+#define KEY2_HASH MURMUR_STRING(key2)
 #else
 #define KEY2TYPE int64_t
 #define KEY2NAME int64
 #define KEY2N i
 #define KEY2STOR int64_t key2
 #define KEY2CPY(m) m->key2=key2
+#define KEY2_HASH MURMUR_INT64(key2)
 #endif
 #define KEY2_EQ_P JOIN(KEY2NAME,eq_p)
-#define KEY2_HASH JOIN(KEY2NAME,hash)
 #endif /* defined(KEY2_TYPE) */
 
 #if defined (KEY3_TYPE)
@@ -124,15 +181,16 @@
 #define KEY3N s
 #define KEY3STOR char key3[MAP_STRING_LENGTH]
 #define KEY3CPY(m) str_copy(m->key3, key3)
+#define KEY3_HASH MURMUR_STRING(key3)
 #else
 #define KEY3TYPE int64_t
 #define KEY3NAME int64
 #define KEY3N i
 #define KEY3STOR int64_t key3
 #define KEY3CPY(m) m->key3=key3
+#define KEY3_HASH MURMUR_INT64(key3)
 #endif
 #define KEY3_EQ_P JOIN(KEY3NAME,eq_p)
-#define KEY3_HASH JOIN(KEY3NAME,hash)
 #endif /* defined(KEY3_TYPE) */
 
 #if defined (KEY4_TYPE)
@@ -144,15 +202,16 @@
 #define KEY4N s
 #define KEY4STOR char key4[MAP_STRING_LENGTH]
 #define KEY4CPY(m) str_copy(m->key4, key4)
+#define KEY4_HASH MURMUR_STRING(key4)
 #else
 #define KEY4TYPE int64_t
 #define KEY4NAME int64
 #define KEY4N i
 #define KEY4STOR int64_t key4
 #define KEY4CPY(m) m->key4=key4
+#define KEY4_HASH MURMUR_INT64(key4)
 #endif
 #define KEY4_EQ_P JOIN(KEY4NAME,eq_p)
-#define KEY4_HASH JOIN(KEY4NAME,hash)
 #endif /* defined(KEY4_TYPE) */
 
 #if defined (KEY5_TYPE)
@@ -164,15 +223,16 @@
 #define KEY5N s
 #define KEY5STOR char key5[MAP_STRING_LENGTH]
 #define KEY5CPY(m) str_copy(m->key5, key5)
+#define KEY5_HASH MURMUR_STRING(key5)
 #else
 #define KEY5TYPE int64_t
 #define KEY5NAME int64
 #define KEY5N i
 #define KEY5STOR int64_t key5
 #define KEY5CPY(m) m->key5=key5
+#define KEY5_HASH MURMUR_INT64(key5)
 #endif
 #define KEY5_EQ_P JOIN(KEY5NAME,eq_p)
-#define KEY5_HASH JOIN(KEY5NAME,hash)
 #endif /* defined(KEY5_TYPE) */
 
 #if defined (KEY6_TYPE)
@@ -184,15 +244,16 @@
 #define KEY6N s
 #define KEY6STOR char key6[MAP_STRING_LENGTH]
 #define KEY6CPY(m) str_copy(m->key6, key6)
+#define KEY6_HASH MURMUR_STRING(key6)
 #else
 #define KEY6TYPE int64_t
 #define KEY6NAME int64
 #define KEY6N i
 #define KEY6STOR int64_t key6
 #define KEY6CPY(m) m->key6=key6
+#define KEY6_HASH MURMUR_INT64(key6)
 #endif
 #define KEY6_EQ_P JOIN(KEY6NAME,eq_p)
-#define KEY6_HASH JOIN(KEY6NAME,hash)
 #endif /* defined(KEY6_TYPE) */
 
 #if defined (KEY7_TYPE)
@@ -204,15 +265,16 @@
 #define KEY7N s
 #define KEY7STOR char key7[MAP_STRING_LENGTH]
 #define KEY7CPY(m) str_copy(m->key7, key7)
+#define KEY7_HASH MURMUR_STRING(key7)
 #else
 #define KEY7TYPE int64_t
 #define KEY7NAME int64
 #define KEY7N i
 #define KEY7STOR int64_t key7
 #define KEY7CPY(m) m->key7=key7
+#define KEY7_HASH MURMUR_INT64(key7)
 #endif
 #define KEY7_EQ_P JOIN(KEY7NAME,eq_p)
-#define KEY7_HASH JOIN(KEY7NAME,hash)
 #endif /* defined(KEY7_TYPE) */
 
 #if defined (KEY8_TYPE)
@@ -224,15 +286,16 @@
 #define KEY8N s
 #define KEY8STOR char key8[MAP_STRING_LENGTH]
 #define KEY8CPY(m) str_copy(m->key8, key8)
+#define KEY8_HASH MURMUR_STRING(key8)
 #else
 #define KEY8TYPE int64_t
 #define KEY8NAME int64
 #define KEY8N i
 #define KEY8STOR int64_t key8
 #define KEY8CPY(m) m->key8=key8
+#define KEY8_HASH MURMUR_INT64(key8)
 #endif
 #define KEY8_EQ_P JOIN(KEY8NAME,eq_p)
-#define KEY8_HASH JOIN(KEY8NAME,hash)
 #endif /* defined(KEY8_TYPE) */
 
 #if defined (KEY9_TYPE)
@@ -244,15 +307,16 @@
 #define KEY9N s
 #define KEY9STOR char key9[MAP_STRING_LENGTH]
 #define KEY9CPY(m) str_copy(m->key9, key9)
+#define KEY9_HASH MURMUR_STRING(key9)
 #else
 #define KEY9TYPE int64_t
 #define KEY9NAME int64
 #define KEY9N i
 #define KEY9STOR int64_t key9
 #define KEY9CPY(m) m->key9=key9
+#define KEY9_HASH MURMUR_INT64(key9)
 #endif
 #define KEY9_EQ_P JOIN(KEY9NAME,eq_p)
-#define KEY9_HASH JOIN(KEY9NAME,hash)
 #endif /* defined(KEY9_TYPE) */
 
 /* Not so many, cowboy! */
@@ -590,25 +654,30 @@ static unsigned int KEYSYM(keycheck) (ALLKEYSD(key))
 	return 1;
 }
 
-static unsigned int KEYSYM(hash) (ALLKEYSD(key))
+static uint32_t KEYSYM(hash) (ALLKEYSD(key)) /* NB: unscaled! */
 {
-	unsigned int hash = KEY1_HASH(key1);
+        /* Open-code 32-bit murmurhash3 */
+        uint32_t len = 0;
+        uint32_t h1 = stap_hash_seed;
+        const uint32_t c1 = 0xcc9e2d51;
+        const uint32_t c2 = 0x1b873593;
+        KEY1_HASH;
 #if KEY_ARITY > 1
-	hash = (hash << 1) ^ KEY2_HASH(key2);
+        KEY2_HASH;
 #if KEY_ARITY > 2
-	hash = (hash << 1) ^ KEY3_HASH(key3);
+        KEY3_HASH;
 #if KEY_ARITY > 3
-	hash = (hash << 1) ^ KEY4_HASH(key4);
+        KEY4_HASH;
 #if KEY_ARITY > 4
-	hash = (hash << 1) ^ KEY5_HASH(key5);
+        KEY5_HASH;
 #if KEY_ARITY > 5
-	hash = (hash << 1) ^ KEY6_HASH(key6);
+        KEY6_HASH;
 #if KEY_ARITY > 6
-	hash = (hash << 1) ^ KEY7_HASH(key7);
+        KEY7_HASH;
 #if KEY_ARITY > 7
-	hash = (hash << 1) ^ KEY8_HASH(key8);
+        KEY8_HASH;
 #if KEY_ARITY > 8
-	hash = (hash << 1) ^ KEY9_HASH(key9);
+        KEY9_HASH;
 #endif
 #endif
 #endif
@@ -617,50 +686,108 @@ static unsigned int KEYSYM(hash) (ALLKEYSD(key))
 #endif
 #endif
 #endif
-	return (unsigned int) (hash % HASH_TABLE_SIZE);
+
+        // finalization
+        h1 ^= len;
+        // fmix32
+        h1 ^= h1 >> 16;
+        h1 *= 0x85ebca6b;
+        h1 ^= h1 >> 13;
+        h1 *= 0xc2b2ae35;
+        h1 ^= h1 >> 16;
+
+        return h1;
 }
 
 
 #if VALUE_TYPE == INT64 || VALUE_TYPE == STRING
-static MAP KEYSYM(_stp_map_new) (unsigned max_entries, int wrap)
+/*
+ * _stp_map_new* ()
+ * @param max_entries (KEY_MAPENTRIES and associated parameter)
+ * @param wrap (KEY_STAT_WRAP)
+ */
+static MAP KEYSYM(_stp_map_new) (int first_arg, ...)
 {
-	MAP m = _stp_map_new (max_entries, wrap,
-			      sizeof(struct KEYSYM(map_node)), -1);
+	int max_entries=0, wrap=0;
+	int arg = first_arg;
+	MAP m;
+	va_list ap;
+
+	va_start (ap, first_arg);
+	do {
+		switch (arg) {
+		case KEY_MAPENTRIES:
+			max_entries = va_arg(ap, int);
+			break;
+		case KEY_STAT_WRAP:
+			wrap = 1;
+		break;
+		default:
+			_stp_warn ("Unknown argument %d\n", arg);
+		}
+		arg = va_arg(ap, int);
+	} while (arg);
+	va_end (ap);
+
+
+	m = _stp_map_new (max_entries, wrap,
+	                  sizeof(struct KEYSYM(map_node)), -1);
 	return m;
 }
 #else
 
 /*
  * _stp_map_new_key1_key2...val (num, wrap, HIST_LINEAR, start, end, interval)
- * _stp_map_new_key1_key2...val (num, wrap, HIST_LOG)
+ * @param num (KEY_MAPENTRIES and associated parameter)
+ * @param wrap (KEY_STAT_WRAP)
+ * @param htype (KEY_HIST_TYPE and associated parameters)
  */ 
-static MAP KEYSYM(_stp_map_new) (unsigned max_entries, int wrap, int htype, ...)
+static MAP KEYSYM(_stp_map_new) (int first_arg, ...)
 {
-	int start=0, stop=0, interval=0;
-	MAP m;
 
-	if (htype == HIST_LINEAR) {
-		va_list ap;
-		va_start (ap, htype);
-		start = va_arg(ap, int);
-		stop = va_arg(ap, int);
-		interval = va_arg(ap, int);
-		va_end (ap);
-	}
+	int start=0, stop=0, interval=0, bit_shift=0;
+	int max_entries=0, wrap=0, htype=0;
+	int arg = first_arg;
+	MAP m;
+	va_list ap;
+
+	va_start (ap, first_arg);
+	do {
+		switch (arg) {
+		case KEY_MAPENTRIES:
+			max_entries = va_arg(ap, int);
+			break;
+		case KEY_STAT_WRAP:
+			wrap = 1;
+			break;
+		case KEY_HIST_TYPE:
+			htype = va_arg(ap, int);
+			if (htype == HIST_LINEAR) {
+				start = va_arg(ap, int);
+				stop = va_arg(ap, int);
+				interval = va_arg(ap, int);
+			}
+			break;
+		default:
+			_stp_warn ("Unknown argument %d\n", arg);
+		}
+		arg = va_arg(ap, int);
+	} while (arg);
+	va_end (ap);
 
 	switch (htype) {
 	case HIST_NONE:
 		m = _stp_map_new_hstat (max_entries, wrap,
-					sizeof(struct KEYSYM(map_node)));
+		                        sizeof(struct KEYSYM(map_node)));
 		break;
 	case HIST_LOG:
 		m = _stp_map_new_hstat_log (max_entries, wrap,
-					    sizeof(struct KEYSYM(map_node)));
+		                            sizeof(struct KEYSYM(map_node)));
 		break;
 	case HIST_LINEAR:
 		m = _stp_map_new_hstat_linear (max_entries, wrap,
-					       sizeof(struct KEYSYM(map_node)),
-					       start, stop, interval);
+		                               sizeof(struct KEYSYM(map_node)),
+		                               start, stop, interval);
 		break;
 	default:
 		_stp_warn ("Unknown histogram type %d\n", htype);
@@ -672,7 +799,7 @@ static MAP KEYSYM(_stp_map_new) (unsigned max_entries, int wrap, int htype, ...)
 
 #endif /* VALUE_TYPE */
 
-static int KEYSYM(__stp_map_set) (MAP map, ALLKEYSD(key), VSTYPE val, int add)
+static inline int KEYSYM(__stp_map_set) (MAP map, ALLKEYSD(key), VSTYPE val, int add, int s1, int s2, int s3, int s4, int s5)
 {
 	unsigned int hv;
 	struct mhlist_head *head;
@@ -685,12 +812,12 @@ static int KEYSYM(__stp_map_set) (MAP map, ALLKEYSD(key), VSTYPE val, int add)
 	if (KEYSYM(keycheck) (ALLKEYS(key)) == 0)
 		return -2;
 
-	hv = KEYSYM(hash) (ALLKEYS(key));
+	hv = KEYSYM(hash) (ALLKEYS(key)) & map->hash_table_mask;
 	head = &map->hashes[hv];
 
 	mhlist_for_each_entry(n, e, head, node.hnode) {
 		if (KEY_EQ_P(n)) {
-			return MAP_SET_VAL(map, n, val, add);
+			return MAP_SET_VAL(map, n, val, add, s1, s2, s3, s4, s5);
 		}
 	}
 	/* key not found */
@@ -698,17 +825,17 @@ static int KEYSYM(__stp_map_set) (MAP map, ALLKEYSD(key), VSTYPE val, int add)
 	if (n == NULL)
 		return -1;
 	KEYCPY(n);
-	return MAP_SET_VAL(map, n, val, 0);
+	return MAP_SET_VAL(map, n, val, 0, s1, s2, s3, s4, s5);
 }
 
 static int KEYSYM(_stp_map_set) (MAP map, ALLKEYSD(key), VSTYPE val)
 {
-	return KEYSYM(__stp_map_set) (map, ALLKEYS(key), val, 0);
+	return KEYSYM(__stp_map_set) (map, ALLKEYS(key), val, 0, 1, 1, 1, 1, 1);
 }
 
 static int KEYSYM(_stp_map_add) (MAP map, ALLKEYSD(key), VSTYPE val)
 {
-	return KEYSYM(__stp_map_set) (map, ALLKEYS(key), val, 1);
+	return KEYSYM(__stp_map_set) (map, ALLKEYS(key), val, 1, 1, 1, 1, 1, 1);
 }
 
 
@@ -722,7 +849,7 @@ static VALTYPE KEYSYM(_stp_map_get) (MAP map, ALLKEYSD(key))
 	if (map == NULL)
 		return NULLRET;
 
-	hv = KEYSYM(hash) (ALLKEYS(key));
+	hv = KEYSYM(hash) (ALLKEYS(key)) & map->hash_table_mask;
 	head = &map->hashes[hv];
 
 	mhlist_for_each_entry(n, e, head, node.hnode) {
@@ -747,7 +874,7 @@ static int KEYSYM(_stp_map_del) (MAP map, ALLKEYSD(key))
 	if (KEYSYM(keycheck) (ALLKEYS(key)) == 0)
 		return -1;
 
-	hv = KEYSYM(hash) (ALLKEYS(key));
+	hv = KEYSYM(hash) (ALLKEYS(key)) & map->hash_table_mask;
 	head = &map->hashes[hv];
 
 	mhlist_for_each_entry(n, e, head, node.hnode) {
@@ -760,6 +887,27 @@ static int KEYSYM(_stp_map_del) (MAP map, ALLKEYSD(key))
 	return 0;
 }
 
+static int KEYSYM(_stp_map_del_hash) (MAP map, unsigned int hv /* scaled */,
+                                      ALLKEYSD(key))
+{
+	struct mhlist_head *head;
+	struct mhlist_node *e;
+	struct KEYSYM(map_node) *n;
+
+	if (map == NULL)
+		return -1;
+
+	head = &map->hashes[hv];
+	mhlist_for_each_entry(n, e, head, node.hnode) {
+		if (KEY_EQ_P(n)) {
+			_new_map_del_node(map, &n->node);
+			return 0;
+		}
+	}
+	/* key not found */
+	return 0;
+}
+
 static int KEYSYM(_stp_map_exists) (MAP map, ALLKEYSD(key))
 {
 	unsigned int hv;
@@ -770,7 +918,7 @@ static int KEYSYM(_stp_map_exists) (MAP map, ALLKEYSD(key))
 	if (map == NULL)
 		return 0;
 
-	hv = KEYSYM(hash) (ALLKEYS(key));
+	hv = KEYSYM(hash) (ALLKEYS(key)) & map->hash_table_mask;
 	head = &map->hashes[hv];
 
 	mhlist_for_each_entry(n, e, head, node.hnode) {
@@ -795,6 +943,7 @@ static int KEYSYM(_stp_map_exists) (MAP map, ALLKEYSD(key))
 #undef KEY1_TYPE
 #undef KEY1STOR
 #undef KEY1CPY
+#undef KEY1_HASH
 
 #undef KEY2NAME
 #undef KEY2N
@@ -802,6 +951,7 @@ static int KEYSYM(_stp_map_exists) (MAP map, ALLKEYSD(key))
 #undef KEY2_TYPE
 #undef KEY2STOR
 #undef KEY2CPY
+#undef KEY2_HASH
 
 #undef KEY3NAME
 #undef KEY3N
@@ -809,6 +959,7 @@ static int KEYSYM(_stp_map_exists) (MAP map, ALLKEYSD(key))
 #undef KEY3_TYPE
 #undef KEY3STOR
 #undef KEY3CPY
+#undef KEY3_HASH
 
 #undef KEY4NAME
 #undef KEY4N
@@ -816,6 +967,7 @@ static int KEYSYM(_stp_map_exists) (MAP map, ALLKEYSD(key))
 #undef KEY4_TYPE
 #undef KEY4STOR
 #undef KEY4CPY
+#undef KEY4_HASH
 
 #undef KEY5NAME
 #undef KEY5N
@@ -823,6 +975,7 @@ static int KEYSYM(_stp_map_exists) (MAP map, ALLKEYSD(key))
 #undef KEY5_TYPE
 #undef KEY5STOR
 #undef KEY5CPY
+#undef KEY5_HASH
 
 #undef KEY6NAME
 #undef KEY6N
@@ -830,6 +983,7 @@ static int KEYSYM(_stp_map_exists) (MAP map, ALLKEYSD(key))
 #undef KEY6_TYPE
 #undef KEY6STOR
 #undef KEY6CPY
+#undef KEY6_HASH
 
 #undef KEY7NAME
 #undef KEY7N
@@ -837,6 +991,7 @@ static int KEYSYM(_stp_map_exists) (MAP map, ALLKEYSD(key))
 #undef KEY7_TYPE
 #undef KEY7STOR
 #undef KEY7CPY
+#undef KEY7_HASH
 
 #undef KEY8NAME
 #undef KEY8N
@@ -844,6 +999,7 @@ static int KEYSYM(_stp_map_exists) (MAP map, ALLKEYSD(key))
 #undef KEY8_TYPE
 #undef KEY8STOR
 #undef KEY8CPY
+#undef KEY8_HASH
 
 #undef KEY9NAME
 #undef KEY9N
@@ -851,6 +1007,7 @@ static int KEYSYM(_stp_map_exists) (MAP map, ALLKEYSD(key))
 #undef KEY9_TYPE
 #undef KEY9STOR
 #undef KEY9CPY
+#undef KEY9_HASH
 
 #undef KEY_ARITY
 #undef ALLKEYS
