@@ -78,6 +78,8 @@ static int _stp_valid_pc_addr(unsigned long addr, struct task_struct *tsk)
 #include "stack-ia64.c"
 #elif defined (__arm__)
 #include "stack-arm.c"
+#elif defined (__mips__)
+#include "stack-mips.c"
 #elif defined (__s390__)
 #include "stack-s390.c"
 #else
@@ -111,7 +113,11 @@ static int print_stack_stack(void *data, char *name)
 	return -1;
 }
 
+#ifdef STAPCONF_STACKTRACE_OPS_INT_ADDRESS
+static int print_stack_address(void *data, unsigned long addr, int reliable)
+#else
 static void print_stack_address(void *data, unsigned long addr, int reliable)
+#endif
 {
 	struct print_stack_data *sdata = data;
 	if (sdata->skip > 0)
@@ -122,6 +128,9 @@ static void print_stack_address(void *data, unsigned long addr, int reliable)
 				NULL);
 		sdata->levels--;
 	}
+#ifdef STAPCONF_STACKTRACE_OPS_INT_ADDRESS
+	return 0;
+#endif
 }
 
 static const struct stacktrace_ops print_stack_ops = {
