@@ -1,6 +1,6 @@
 /* -*- linux-c -*- 
  * common stats functions for aggragations and maps
- * Copyright (C) 2005, 2006, 2007, 2008 Red Hat Inc.
+ * Copyright (C) 2005-2016 Red Hat Inc.
  *
  * This file is part of systemtap, and is free software.  You can
  * redistribute it and/or modify it under the terms of the GNU General
@@ -261,10 +261,10 @@ static void _stp_stat_print_histogram_buf(char *buf, size_t size, Hist st,
 				val_prefix = "<";
 			} else if (i == st->buckets-1) {
 				/* overflow */
-				val = st->start + (i - 2) * st->interval;
+				val = st->start + (int64_t)(i - 2) * st->interval;
 				val_prefix = ">";
 			} else
-				val = st->start + (i - 1) * st->interval;
+				val = st->start + (int64_t)(i - 1) * st->interval;
 		} else
 			val = _stp_bucket_to_val(i);
 
@@ -288,20 +288,41 @@ static void _stp_stat_print_histogram(Hist st, stat_data *sd)
 	_stp_print_flush();
 }
 
-static void __stp_stat_add(Hist st, stat_data *sd, int64_t val)
+static inline void __stp_stat_add(Hist st, stat_data *sd, int64_t val,
+                                  int stat_op_count, int stat_op_sum, int stat_op_min,
+				  int stat_op_max, int stat_op_variance)
 {
 	int n;
+	int delta = 0;
+
+	sd->shift = st->bit_shift;
+	sd->stat_ops = st->stat_ops;
 	if (sd->count == 0) {
 		sd->count = 1;
 		sd->sum = sd->min = sd->max = val;
+		sd->avg_s = val << sd->shift;
+		sd->_M2 = 0;
 	} else {
-		sd->count++;
-		sd->sum += val;
-		if (val > sd->max)
+		if(stat_op_count)
+			sd->count++;
+		if(stat_op_sum)
+			sd->sum += val;
+		if (stat_op_min && (val > sd->max))
 			sd->max = val;
-		if (val < sd->min)
+		if (stat_op_max && (val < sd->min))
 			sd->min = val;
+		/*
+		 * Below, we use Welford's online algorithm for computing variance.
+		 * https://en.wikipedia.org/wiki/Algorithms_for_calculating_variance
+		 */
+		if (stat_op_variance) {
+		    delta = (val << sd->shift) - sd->avg_s;
+		    sd->avg_s += _stp_div64(NULL, delta, sd->count);
+		    sd->_M2 += delta * ((val << sd->shift) - sd->avg_s);
+		    sd->variance_s = (sd->count < 2) ? -1 : _stp_div64(NULL, sd->_M2, (sd->count - 1));
+		}
 	}
+
 	switch (st->type) {
 	case HIST_LOG:
 		n = _stp_val_to_bucket (val);
