@@ -91,7 +91,7 @@ __stp_get_freq(void)
     // If we can get the actual frequency of HW counter, we use it.
 #if defined (__ia64__)
     return local_cpu_data->itc_freq / 1000;
-#elif defined (__s390__) || defined (__s390x__) || defined (__arm__) || defined (__aarch64__)
+#elif defined (__s390__) || defined (__s390x__) || defined (__arm__)
     // We don't need to find the cpu freq on s390 as the 
     // TOD clock is always a fix freq. (see: POO pg 4-36.)
     return 0;
@@ -99,7 +99,7 @@ __stp_get_freq(void)
     return tsc_khz;
 #elif (defined (__i386__) || defined (__x86_64__)) && defined(STAPCONF_CPU_KHZ)
     return cpu_khz;
-#else /* __i386__ || __x86_64__ */
+#else /* __i386__ || __x86_64__ || __aarch64__ */
     // If we don't know the actual frequency, we estimate it.
     cycles_t beg, mid, end;
     beg = get_cycles(); barrier();
@@ -130,7 +130,7 @@ __stp_ktime_get_real_ts(struct timespec *ts)
 static stp_time_t*
 __stp_time_local_update(void)
 {
-    unsigned long flags;
+    unsigned long flags, flags2;
     stp_time_t *time;
     struct timespec ts;
     int64_t ns;
@@ -143,10 +143,15 @@ __stp_time_local_update(void)
     ns = (NSEC_PER_SEC * (int64_t)ts.tv_sec) + ts.tv_nsec;
     time = per_cpu_ptr(stp_time, smp_processor_id());
 
-    write_seqlock(&time->lock);
+    // Why are we calling write_seqlock_irqsave() here instead of just
+    // write_seqlock() (especially since we've already disabled
+    // interrupts using local_irq_save())? The answer is that
+    // write_seqlock_irqsave() does the correct wait for when
+    // interrupts are disabled.
+    write_seqlock_irqsave(&time->lock, flags2);
     time->base_ns = ns;
     time->base_cycles = cycles;
-    write_sequnlock(&time->lock);
+    write_sequnlock_irqrestore(&time->lock, flags2);
 
     local_irq_restore(flags);
 
@@ -171,7 +176,10 @@ __stp_time_timer_callback(unsigned long val)
     stp_time_t *time =__stp_time_local_update();
     (void) val;
 
-    /* PR6481: reenable IRQs before resetting the timer.
+    /* PR6481: make sure IRQs are enabled before resetting the timer
+       (IRQs are disabled and then reenabled in
+       __stp_time_local_update()).
+
        XXX: The worst that can probably happen is that we get
 	    two consecutive timer resets.  */
 
@@ -304,6 +312,7 @@ _stp_init_time(void)
 {
     int cpu, ret = 0;
 
+    might_sleep();
     _stp_kill_time();
 
     stp_time = _stp_alloc_percpu(sizeof(stp_time_t));
