@@ -12,6 +12,7 @@
 #ifndef _STP_SYMBOLS_C_
 #define _STP_SYMBOLS_C_
 #include "../sym.h"
+#include "relay_compat.h"
 
 #ifndef KERN_CONT
 #define KERN_CONT	""
@@ -249,7 +250,12 @@ static int _stp_module_update_self (void)
 		}
 		else if (!strcmp(".symtab", attr->name)) {
 #ifdef STAPCONF_MOD_KALLSYMS
-			struct mod_kallsyms *kallsyms = rcu_dereference_sched(mod->kallsyms);
+			struct mod_kallsyms *kallsyms;
+
+			rcu_read_lock_sched();
+			kallsyms = rcu_dereference_sched(mod->kallsyms);
+			rcu_read_unlock_sched();
+
 			if (attr->address == (unsigned long) kallsyms->symtab)
 				_stp_module_self.sections[0].size =
 					kallsyms->num_symtab * sizeof(kallsyms->symtab[0]);
@@ -322,7 +328,7 @@ static int _stp_module_panic_notifier (struct notifier_block *nb, unsigned long
                 int printed;
                 int first_iteration;
 
-                sub_buf = _stp_relay_data.rchan->buf[i];
+                sub_buf = _stp_get_rchan_subbuf(_stp_relay_data.rchan->buf, i);
 
                 /* Set our pointer to the beginning of the channel buffer */
                 subbuf_start = (char *)sub_buf->start;
