@@ -1,8 +1,8 @@
-/* -*- linux-c -*-
+/* -*- linux-c -*- 
  * transport.c - stp transport functions
  *
  * Copyright (C) IBM Corporation, 2005
- * Copyright (C) Red Hat Inc, 2005-2015
+ * Copyright (C) Red Hat Inc, 2005-2014
  * Copyright (C) Intel Corporation, 2006
  *
  * This file is part of systemtap, and is free software.  You can
@@ -21,11 +21,6 @@
 #include <linux/delay.h>
 #include <linux/mutex.h>
 #include "../uidgid_compatibility.h"
-#ifdef STAPCONF_MODULE_TRACEPOINT
-#include <trace/events/module.h>
-#include "../linux/stp_tracepoint.h"
-#endif
-
 
 static int _stp_exit_flag = 0;
 
@@ -41,12 +36,14 @@ static atomic_t _stp_ctl_attached = ATOMIC_INIT(0);
 static pid_t _stp_target = 0;
 static int _stp_probes_started = 0;
 
-/* _stp_transport_mutext guards _stp_start_called and _stp_exit_called.
-   We only want to do the startup and exit sequences once.  Note that
-   these indicate the respective process starting, not their conclusion. */
-static int _stp_start_called = 0;
-static int _stp_exit_called = 0;
-static DEFINE_MUTEX(_stp_transport_mutex);
+#define _STP_TS_UNINITIALIZED	0
+#define _STP_TS_START_CALLED	1
+#define _STP_TS_START_FINISHED	2
+#define _STP_TS_EXIT_CALLED	3
+
+/* We only want to do the startup and exit sequences once, so the
+ * transport state flag is atomic. */
+static atomic_t _stp_transport_state = ATOMIC_INIT(_STP_TS_UNINITIALIZED);
 
 #ifndef STP_CTL_TIMER_INTERVAL
 /* ctl timer interval in jiffies (default 20 ms) */
@@ -88,11 +85,6 @@ static void systemtap_module_exit(void);
 static int systemtap_module_init(void);
 
 static int _stp_module_notifier_active = 0;
-#ifdef STAPCONF_MODULE_TRACEPOINT
-/* callbacks in runtime/transport/symbols.c */
-static void _stp_module_load_tp(void *data, struct module* mod);
-static void _stp_module_free_tp(void *data, struct module* mod);
-#endif
 static int _stp_module_notifier (struct notifier_block * nb,
                                  unsigned long val, void *data);
 static struct notifier_block _stp_module_notifier_nb = {
@@ -133,11 +125,16 @@ static struct timer_list _stp_ctl_work_timer;
 static void _stp_handle_start(struct _stp_msg_start *st)
 {
 	int handle_startup;
+#if defined(CONFIG_USER_NS)
+	struct pid *_upid = NULL;
+	struct task_struct *_utask = NULL;
+#endif
 
         // protect against excessive or premature startup
-	handle_startup = (! _stp_start_called && ! _stp_exit_called);
-	_stp_start_called = 1;
-
+	handle_startup = (atomic_cmpxchg(&_stp_transport_state,
+					 _STP_TS_UNINITIALIZED,
+					 _STP_TS_START_CALLED)
+			  == _STP_TS_UNINITIALIZED);
 	if (handle_startup) {
 		dbug_trans(1, "stp_handle_start\n");
 
@@ -159,6 +156,30 @@ static void _stp_handle_start(struct _stp_msg_start *st)
 #endif
 
 		_stp_target = st->target;
+
+#if defined(CONFIG_USER_NS)
+                rcu_read_lock();
+                _upid = find_vpid(_stp_target);
+                if (_upid)
+                {
+                    _utask = pid_task(_upid, PIDTYPE_PID);
+                    if (_utask)
+                    {
+                        #ifdef DEBUG_UPROBES
+                        _stp_dbug(__FUNCTION__,__LINE__, "translating vpid %d to pid %d\n", _stp_target, _utask->pid);
+                        #endif
+                        _stp_target = _utask->pid;
+                    }
+                }
+
+                #ifdef DEBUG_UPROBES
+                if (!_upid || !_utask)
+                    _stp_dbug(__FUNCTION__,__LINE__, "cannot map pid %d to host namespace pid\n", _stp_target);
+                #endif
+
+                rcu_read_unlock();
+#endif
+
 		st->res = systemtap_module_init();
 		if (st->res == 0) {
 			_stp_probes_started = 1;
@@ -168,29 +189,11 @@ static void _stp_handle_start(struct _stp_msg_start *st)
                            failed: something nasty has happened, and
                            we want no further probing started.  PR16766 */
                         if (!_stp_module_notifier_active) {
-#ifdef STAPCONF_MODULE_TRACEPOINT
-                                int rc0 = STP_TRACE_REGISTER(module_load, & _stp_module_load_tp);
-                                if (rc0)
-                                        _stp_warn ("Cannot register module load tracepoint (%d)\n", rc0);
-                                else {
-                                        int rc1 = STP_TRACE_REGISTER(module_free, & _stp_module_free_tp);
-                                        if (rc1) {
-                                                _stp_warn ("Cannot register module free tracepoint (%d)\n", rc1);
-                                                STP_TRACE_UNREGISTER(module_load, & _stp_module_load_tp);
-                                        } else {
-#endif
-                                                int rc = register_module_notifier(& _stp_module_notifier_nb);
-                                                if (rc == 0)
-                                                        _stp_module_notifier_active = 1;
-                                                else {
-                                                        _stp_warn ("Cannot register module notifier (%d)\n", rc);
-#ifdef STAPCONF_MODULE_TRACEPOINT
-                                                        STP_TRACE_UNREGISTER(module_load, & _stp_module_load_tp);
-                                                        STP_TRACE_UNREGISTER(module_free, & _stp_module_free_tp);
-                                                }
-                                        }
-#endif
-                                }
+                                int rc = register_module_notifier(& _stp_module_notifier_nb);
+                                if (rc == 0)
+                                        _stp_module_notifier_active = 1;
+                                else
+                                        _stp_warn ("Cannot register module notifier (%d)\n", rc);
                         }
                 }
 
@@ -203,6 +206,11 @@ static void _stp_handle_start(struct _stp_msg_start *st)
 #if STP_TRANSPORT_VERSION == 2
 		atomic_notifier_chain_register(&panic_notifier_list, &_stp_module_panic_notifier_nb);
 #endif
+
+		/* If we're here, we know that _stp_transport_state
+		 * is _STP_TS_START_CALLED. Go ahead and set it to
+		 * _STP_TS_START_FINISHED. */
+		atomic_set(&_stp_transport_state, _STP_TS_START_FINISHED);
 	}
 }
 
@@ -216,9 +224,10 @@ static void _stp_cleanup_and_exit(int send_exit)
 	int handle_exit;
 
         // protect against excessive or premature cleanup
-	handle_exit = (_stp_start_called && ! _stp_exit_called);
-	_stp_exit_called = 1;
-
+	handle_exit = (atomic_cmpxchg(&_stp_transport_state,
+				      _STP_TS_START_FINISHED,
+				      _STP_TS_EXIT_CALLED)
+		       == _STP_TS_START_FINISHED);
 	if (handle_exit) {
 		int failures;
 
@@ -226,12 +235,7 @@ static void _stp_cleanup_and_exit(int send_exit)
 
 	        /* Unregister the module notifier. */
 	        if (_stp_module_notifier_active) {
-                        int rc;
-#ifdef STAPCONF_MODULE_TRACEPOINT
-                        STP_TRACE_UNREGISTER(module_load, & _stp_module_load_tp);
-                        STP_TRACE_UNREGISTER(module_free, & _stp_module_free_tp);
-#endif
-                        rc = unregister_module_notifier(& _stp_module_notifier_nb);
+                        int rc = unregister_module_notifier(& _stp_module_notifier_nb);
                         if (rc)
                                 _stp_warn("module_notifier unregister error %d", rc);
 	                _stp_module_notifier_active = 0;
@@ -367,7 +371,7 @@ static void _stp_ctl_work_callback(unsigned long val)
  *	_stp_transport_close - close ctl and relayfs channels
  *
  *	This is called automatically when the module is unloaded.
- *
+ *     
  */
 static void _stp_transport_close(void)
 {
@@ -384,7 +388,7 @@ static void _stp_transport_close(void)
 
 /**
  * _stp_transport_init() is called from the module initialization.
- *   It does the bare minimum to exchange commands with staprun
+ *   It does the bare minimum to exchange commands with staprun 
  */
 static int _stp_transport_init(void)
 {
@@ -443,13 +447,6 @@ static int _stp_transport_init(void)
 #endif
 #endif
 
-
-#ifdef RELAY_GUEST
-	/* Guest scripts use relay only for reporting warnings and errors */
-	_stp_subbuf_size = 65536;
-	_stp_nsubbufs = 2;
-#endif
-
 	if (_stp_bufsize) {
 		unsigned size = _stp_bufsize * 1024 * 1024;
 		_stp_subbuf_size = 65536;
@@ -502,20 +499,29 @@ err0:
 
 static inline void _stp_lock_inode(struct inode *inode)
 {
+#ifdef STAPCONF_INODE_RWSEM
+	inode_lock(inode);
+#else
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,16)
+	might_sleep();
 	mutex_lock(&inode->i_mutex);
 #else
 	down(&inode->i_sem);
 #endif
+#endif
 }
 
 static inline void _stp_unlock_inode(struct inode *inode)
 {
+#ifdef STAPCONF_INODE_RWSEM
+	inode_unlock(inode);
+#else
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,16)
 	mutex_unlock(&inode->i_mutex);
 #else
 	up(&inode->i_sem);
 #endif
+#endif
 }
 
 static struct dentry *_stp_lockfile = NULL;
@@ -644,7 +650,7 @@ static struct dentry *_stp_get_module_dir(void)
 static int _stp_transport_fs_init(const char *module_name)
 {
 	struct dentry *root_dir;
-
+    
 	dbug_trans(1, "entry\n");
 	if (module_name == NULL)
 		return -1;
