@@ -1,5 +1,5 @@
 // -*- C++ -*-
-// Copyright (C) 2005-2015 Red Hat Inc.
+// Copyright (C) 2005-2016 Red Hat Inc.
 //
 // This file is part of systemtap, and is free software.  You can
 // redistribute it and/or modify it under the terms of the GNU General
@@ -33,6 +33,15 @@ extern "C" {
 #include "util.h"
 #include "stringtable.h"
 
+/* statistical operations used with a global */
+#define STAT_OP_NONE      1 << 0
+#define STAT_OP_COUNT     1 << 1
+#define STAT_OP_SUM       1 << 2
+#define STAT_OP_MIN       1 << 3
+#define STAT_OP_MAX       1 << 4
+#define STAT_OP_AVG       1 << 5
+#define STAT_OP_VARIANCE  1 << 6
+
 // forward decls for all referenced systemtap types
 class stap_hash;
 class match_node;
@@ -42,8 +51,7 @@ struct token;
 struct functiondecl;
 struct derived_probe;
 struct be_derived_probe_group;
-struct dwarf_derived_probe_group;
-struct kprobe_derived_probe_group;
+struct generic_kprobe_derived_probe_group;
 struct hwbkpt_derived_probe_group;
 struct perf_derived_probe_group;
 struct uprobe_derived_probe_group;
@@ -58,6 +66,7 @@ struct tracepoint_derived_probe_group;
 struct hrtimer_derived_probe_group;
 struct procfs_derived_probe_group;
 struct dynprobe_derived_probe_group;
+struct python_derived_probe_group;
 struct embeddedcode;
 struct stapdfa;
 class translator_output;
@@ -73,12 +82,14 @@ struct statistic_decl
 {
   statistic_decl()
     : type(none),
-      linear_low(0), linear_high(0), linear_step(0)
+      linear_low(0), linear_high(0), linear_step(0), bit_shift(0), stat_ops(0)
   {}
   enum { none, linear, logarithmic } type;
   int64_t linear_low;
   int64_t linear_high;
   int64_t linear_step;
+  int bit_shift;
+  int stat_ops;
   bool operator==(statistic_decl const & other)
   {
     return type == other.type
@@ -157,6 +168,7 @@ public:
   int include_arg_start;
   std::vector<std::string> c_macros;
   std::vector<std::string> args;
+  std::vector<bool> used_args;
   std::vector<std::string> kbuildflags; // -B var=val
   std::vector<std::string> globalopts; // -G var=val
   std::vector<std::string> modinfos; // --modinfo tag=value
@@ -166,6 +178,7 @@ public:
   std::string kernel_base_release;
   std::string kernel_build_tree;
   std::string kernel_source_tree;
+  std::vector<std::string> kernel_extra_cflags; 
   std::map<interned_string,interned_string> kernel_config;
   std::set<interned_string> kernel_exports;
   std::set<interned_string> kernel_functions;
@@ -213,11 +226,13 @@ public:
   std::string uprobes_path;
   std::string uprobes_hash;
   bool load_only; // flight recorder mode
-  bool omit_werror;
   privilege_t privilege;
   bool privilege_set;
   bool systemtap_v_check;
   bool tmpdir_opt_set;
+  bool monitor;
+  int monitor_interval;
+  int timeout; // in ms
 
   enum
    { dump_none,               // no dumping requested
@@ -235,6 +250,8 @@ public:
   bool suppress_handler_errors;
   bool suppress_time_limits;
   bool color_errors;
+  bool interactive_mode;
+  bool pass_1a_complete;
 
   enum { color_never, color_auto, color_always } color_mode;
   enum { prologue_searching_never, prologue_searching_auto, prologue_searching_always } prologue_searching_mode;
@@ -331,6 +348,7 @@ public:
   std::vector<stapfile*> files;
   std::vector<vardecl*> globals;
   std::map<std::string,functiondecl*> functions;
+  std::map<std::string,unsigned> overload_count;
   // probe counter name -> probe associated with counter
   std::vector<std::pair<std::string,std::string> > perf_counters;
   std::vector<derived_probe*> probes; // see also *_probes groups below
@@ -351,8 +369,7 @@ public:
   // Every probe in these groups must also appear in the
   // session.probes vector.
   be_derived_probe_group* be_derived_probes;
-  dwarf_derived_probe_group* dwarf_derived_probes;
-  kprobe_derived_probe_group* kprobe_derived_probes;
+  generic_kprobe_derived_probe_group* generic_kprobe_derived_probes;
   hwbkpt_derived_probe_group* hwbkpt_derived_probes;
   perf_derived_probe_group* perf_derived_probes;
   uprobe_derived_probe_group* uprobe_derived_probes;
@@ -367,6 +384,7 @@ public:
   hrtimer_derived_probe_group* hrtimer_derived_probes;
   procfs_derived_probe_group* procfs_derived_probes;
   dynprobe_derived_probe_group* dynprobe_derived_probes;
+  python_derived_probe_group* python_derived_probes;
 
   // NB: It is very important for all of the above (and below) fields
   // to be cleared in the systemtap_session ctor (session.cxx).
@@ -447,6 +465,8 @@ public:
   bool is_user_file (const std::string& name);
   bool is_primary_probe (derived_probe *dp);
 
+  void clear_script_data();
+
   // NB: It is very important for all of the above (and below) fields
   // to be cleared in the systemtap_session ctor (session.cxx).
 };
