@@ -27,6 +27,7 @@
 #include <climits>
 #include <iostream>
 #include <map>
+#include <thread>
 
 extern "C" {
 #include <unistd.h>
@@ -104,10 +105,11 @@ extern int optind;
 static bool use_db_password;
 static unsigned short port;
 static long max_threads;
+static size_t max_uncompressed_req_size;
+static size_t max_compressed_req_size;
 static string cert_db_path;
 static string stap_options;
-static string uname_r;
-static string kernel_build_tree;
+static map<string,string> kernel_build_tree; // Kernel version -> build tree
 static string arch;
 static string cert_serial_number;
 static string B_options;
@@ -180,16 +182,9 @@ static void
 process_r (const string &arg)
 {
   if (arg[0] == '/') // fully specified path
-    {
-      kernel_build_tree = arg;
-      uname_r = kernel_release_from_build_tree (arg);
-    }
+    kernel_build_tree.insert({kernel_release_from_build_tree(arg), arg});
   else
-    {
-      kernel_build_tree = "/lib/modules/" + arg + "/build";
-      uname_r = arg;
-    }
-  stap_options += " -r " + arg; // Pass the argument to stap directly.
+    kernel_build_tree.insert({arg, "/lib/modules/" + arg + "/build" });
 }
 
 static void
@@ -208,19 +203,24 @@ parse_options (int argc, char **argv)
     {
       char *num_endptr;
       long port_tmp;
+      long maxsize_tmp;
       // NB: The values of these enumerators must not conflict with the values of ordinary
       // characters, since those are returned by getopt_long for short options.
       enum {
 	LONG_OPT_PORT = 256,
 	LONG_OPT_SSL,
 	LONG_OPT_LOG,
-	LONG_OPT_MAXTHREADS
+	LONG_OPT_MAXTHREADS,
+        LONG_OPT_MAXREQSIZE = 254,
+        LONG_OPT_MAXCOMPRESSEDREQ = 255 /* need to set a value otherwise there are conflicts */
       };
       static struct option long_options[] = {
         { "port", 1, NULL, LONG_OPT_PORT },
         { "ssl", 1, NULL, LONG_OPT_SSL },
         { "log", 1, NULL, LONG_OPT_LOG },
         { "max-threads", 1, NULL, LONG_OPT_MAXTHREADS },
+        { "max-request-size", 1, NULL, LONG_OPT_MAXREQSIZE},
+        { "max-compressed-request", 1, NULL, LONG_OPT_MAXCOMPRESSEDREQ},
         { NULL, 0, NULL, 0 }
       };
       int grc = getopt_long (argc, argv, "a:B:D:I:kPr:R:", long_options, NULL);
@@ -280,6 +280,24 @@ parse_options (int argc, char **argv)
 	    fatal (_F("%s: invalid entry: max threads must not be negative '--max-threads=%s'",
 		      argv[0], optarg));
 	  break;
+        case LONG_OPT_MAXREQSIZE:
+          maxsize_tmp =  strtoul(optarg, &num_endptr, 0); // store as a long for now
+	  if (*num_endptr != '\0')
+	    fatal (_F("%s: cannot parse number '--max-request-size=%s'", argv[0], optarg));
+          else if (maxsize_tmp < 1)
+	    fatal (_F("%s: invalid entry: max (uncompressed) request size must be greater than 0 '--max-request-size=%s'",
+		      argv[0], optarg));
+          max_uncompressed_req_size = (size_t) maxsize_tmp; // convert the long to an unsigned
+          break;
+        case LONG_OPT_MAXCOMPRESSEDREQ:
+          maxsize_tmp =  strtoul(optarg, &num_endptr, 0); // store as a long for now
+	  if (*num_endptr != '\0')
+	    fatal (_F("%s: cannot parse number '--max-compressed-request=%s'", argv[0], optarg));
+          else if (maxsize_tmp < 1)
+	    fatal (_F("%s: invalid entry: max compressed request size must be greater than 0 '--max-compressed-request=%s'",
+		      argv[0], optarg));
+          max_compressed_req_size = (size_t) maxsize_tmp; // convert the long to an unsigned
+          break;
 	case '?':
 	  // Invalid/unrecognized option given. Message has already been issued.
 	  break;
@@ -380,8 +398,8 @@ mok_dir_valid_p (string mok_fingerprint, bool verbose)
 
 	  // If the filesystem doesn't support d_type, we'll have to
 	  // call stat().
-	  stat((mok_dir + "/" + direntp->d_name).c_str (), &tmpstat);
-	  if (S_ISREG(tmpstat.st_mode))
+	  int rc = stat((mok_dir + "/" + direntp->d_name).c_str (), &tmpstat);
+	  if (rc == 0 && S_ISREG(tmpstat.st_mode))
 	      reg_file = true;
         }
       
@@ -479,8 +497,8 @@ get_server_mok_fingerprints(vector<string> &mok_fingerprints, bool verbose,
               // If the filesystem doesn't support d_type, we'll have to
               // call stat().
               struct stat tmpstat;
-              stat((mok_path + "/" + direntp->d_name).c_str (), &tmpstat);
-              if (!S_ISDIR(tmpstat.st_mode))
+              int rc = stat((mok_path + "/" + direntp->d_name).c_str (), &tmpstat);
+              if (rc || !S_ISDIR(tmpstat.st_mode))
                 continue;
             }
           else
@@ -607,11 +625,11 @@ create_services (AvahiClient *c)
   if (! avahi_group)
     {
       if (! (avahi_group = avahi_entry_group_new (c, entry_group_callback, NULL)))
-	{
-	  server_error (_F("avahi_entry_group_new () failed: %s",
-			   avahi_strerror (avahi_client_errno (c))));
-	  return;
-	}
+        {
+          server_error (_F("avahi_entry_group_new () failed: %s",
+          avahi_strerror (avahi_client_errno (c))));
+          return;
+        }
     }
   else
     avahi_entry_group_reset(avahi_group);
@@ -619,89 +637,95 @@ create_services (AvahiClient *c)
   // Contruct the information needed for our service.
   log (_F("Adding Avahi service '%s'", avahi_service_name));
 
-  // Create the txt tags that will be registered with our service.
-  string sysinfo = "sysinfo=" + uname_r + " " + arch;
-  string certinfo = "certinfo=" + cert_serial_number;
-  string version = string ("version=") + CURRENT_CS_PROTOCOL_VERSION;;
-  string optinfo = "optinfo=";
-  string separator;
-  // These option strings already have a leading space.
-  if (! R_option.empty ())
-    {
-      optinfo += R_option.substr(1);
-      separator = " ";
-    }
-  if (! B_options.empty ())
-    {
-      optinfo += separator + B_options.substr(1);
-      separator = " ";
-    }
-  if (! D_options.empty ())
-    {
-      optinfo += separator + D_options.substr(1);
-      separator = " ";
-    }
-  if (! I_options.empty ())
-    optinfo += separator + I_options.substr(1);
-
-  // Create an avahi string list with the info we have so far.
-  vector<string> mok_fingerprints;
-  AvahiStringList *strlst = avahi_string_list_new(sysinfo.c_str (),
-						  optinfo.c_str (),
-						  version.c_str (),
-						  certinfo.c_str (), NULL);
-  if (strlst == NULL)
-    {
-      server_error (_("Failed to allocate string list"));
-      goto fail;
-    }
-
-  // Add server MOK info, if available.
-  get_server_mok_fingerprints (mok_fingerprints, true, false);
-  if (! mok_fingerprints.empty())
-    {
-      vector<string>::const_iterator it;
-      for (it = mok_fingerprints.begin(); it != mok_fingerprints.end(); it++)
-        {
-	  string tmp = "mok_info=" + *it;
-	  strlst = avahi_string_list_add(strlst, tmp.c_str ());
-	  if (strlst == NULL)
-	    {
-	      server_error (_("Failed to add a string to the list"));
-	      goto fail;
-	    }
-	}
-    }
-
-  // We will now add our service to the entry group.
-  // Loop until no collisions.
+  AvahiStringList *strlst = NULL;
   int ret;
-  for (;;) {
-    ret = avahi_entry_group_add_service_strlst (avahi_group,
-						AVAHI_IF_UNSPEC,
-						AVAHI_PROTO_UNSPEC,
-						(AvahiPublishFlags)0,
-						avahi_service_name,
-						avahi_service_tag,
-						NULL, NULL, port, strlst);
-    if (ret == AVAHI_OK)
-      break; // success!
+  for (auto it = kernel_build_tree.cbegin(); it != kernel_build_tree.cend(); ++it)
+    {
+      // Create the txt tags that will be registered with our service.
+      string sysinfo = "sysinfo=" + it->first + " "+ arch;
+      string certinfo = "certinfo=" + cert_serial_number;
+      string version = string ("version=") + CURRENT_CS_PROTOCOL_VERSION;;
+      string optinfo = "optinfo=";
+      string separator;
+      // These option strings already have a leading space.
+      if (! R_option.empty ())
+        {
+          optinfo += R_option.substr(1);
+          separator = " ";
+        }
+      if (! B_options.empty ())
+        {
+          optinfo += separator + B_options.substr(1);
+          separator = " ";
+        }
+      if (! D_options.empty ())
+        {
+          optinfo += separator + D_options.substr(1);
+          separator = " ";
+        }
+      if (! I_options.empty ())
+        optinfo += separator + I_options.substr(1);
 
-    if (ret == AVAHI_ERR_COLLISION)
-      {
-	// A service name collision with a local service happened.
-	// Pick a new name.
-	if (rename_service () < 0) {
-	  // Too many collisions. Message already issued.
-	  goto fail;
-	}
-	continue; // try again.
+      // Create an avahi string list with the info we have so far.
+      vector<string> mok_fingerprints;
+      strlst = avahi_string_list_new(sysinfo.c_str (),
+                                     optinfo.c_str (),
+                                     version.c_str (),
+                                     certinfo.c_str (), NULL);
+      if (strlst == NULL)
+        {
+          server_error (_("Failed to allocate string list"));
+          goto fail;
+        }
+
+      // Add server MOK info, if available.
+      get_server_mok_fingerprints (mok_fingerprints, true, false);
+      if (! mok_fingerprints.empty())
+        {
+          for (auto it = mok_fingerprints.cbegin(); it != mok_fingerprints.cend(); it++)
+            {
+              string tmp = "mok_info=" + *it;
+              strlst = avahi_string_list_add(strlst, tmp.c_str ());
+              if (strlst == NULL)
+                {
+                  server_error (_("Failed to add a string to the list"));
+                  goto fail;
+                }
+            }
+        }
+
+      // We will now add our service to the entry group.
+      // Loop until no collisions.
+      for (;;) {
+        ret = avahi_entry_group_add_service_strlst (avahi_group,
+                                                    AVAHI_IF_UNSPEC,
+                                                    AVAHI_PROTO_UNSPEC,
+                                                    (AvahiPublishFlags)0,
+                                                    avahi_service_name,
+                                                    avahi_service_tag,
+                                                    NULL, NULL, port, strlst);
+        if (ret == AVAHI_OK)
+          break; // success!
+
+        if (ret == AVAHI_ERR_COLLISION)
+          {
+            // A service name collision with a local service happened.
+            // Pick a new name.
+            if (rename_service () < 0) {
+              // Too many collisions. Message already issued.
+              goto fail;
+            }
+            continue; // try again.
+          }
+
+          server_error (_F("Failed to add %s service: %s",
+              avahi_service_tag, avahi_strerror (ret)));
+          goto fail;
       }
 
-      server_error (_F("Failed to add %s service: %s",
-		       avahi_service_tag, avahi_strerror (ret)));
-      goto fail;
-  }
+      avahi_string_list_free(strlst);
+      strlst = NULL;
+    }
 
   // Tell the server to register the service.
   if ((ret = avahi_entry_group_commit (avahi_group)) < 0)
@@ -709,8 +733,6 @@ create_services (AvahiClient *c)
       server_error (_F("Failed to commit avahi entry group: %s", avahi_strerror (ret)));
       goto fail;
     }
-
-  avahi_string_list_free(strlst);
   return;
 
  fail:
@@ -781,7 +803,7 @@ client_callback (AvahiClient *c, AvahiClientState state, AVAHI_GCC_UNUSED void *
 }
 
 static void
-inotify_callback (AvahiWatch *w, int fd, AvahiWatchEvent event, void *userdata)
+inotify_callback (AvahiWatch *, int fd, AvahiWatchEvent, void *)
 {
   struct inotify_event in_events[10];
   ssize_t rc;
@@ -960,12 +982,13 @@ initialize (int argc, char **argv) {
   // Initial values.
   use_db_password = false;
   port = 0;
-  max_threads = sysconf( _SC_NPROCESSORS_ONLN ); // Default to number of processors
+  max_threads = thread::hardware_concurrency(); // Default to number of processors
+  max_uncompressed_req_size = 50000; // 50 KB: default max uncompressed request size
+  max_compressed_req_size = 5000; // 5 KB: default max compressed request size
   keep_temp = false;
   struct utsname utsname;
   uname (& utsname);
-  uname_r = utsname.release;
-  kernel_build_tree = "/lib/modules/" + uname_r + "/build";
+  kernel_build_tree.insert({utsname.release, "/lib/modules/" + string(utsname.release) + "/build"});
   arch = normalize_machine (utsname.machine);
 
   // Parse the arguments. This also starts the server log, if any, and should be done before
@@ -1024,7 +1047,6 @@ readDataFromSocket(PRFileDesc *sslSocket, const char *requestFileName)
   char        buffer[READ_BUFFER_SIZE];
 
   // Read the number of bytes to be received.
-  /* XXX: impose a limit to prevent disk space consumption DoS */
   numBytesRead = PR_Read_Complete (sslSocket, & numBytesExpected,
 				   (PRInt32)sizeof (numBytesExpected));
   if (numBytesRead == 0) /* EOF */
@@ -1047,6 +1069,13 @@ readDataFromSocket(PRFileDesc *sslSocket, const char *requestFileName)
   if (numBytesExpected == 0)
     return 0;
 
+  /* Impose a limit to prevent disk space consumption DoS */
+  if (numBytesExpected > (PRInt32) max_compressed_req_size)
+    {
+      server_error (_("Error size of (compressed) request file is too large"));
+      goto done;
+    }
+
   /* Open the output file.  */
   local_file_fd = PR_Open(requestFileName, PR_WRONLY | PR_CREATE_FILE | PR_TRUNCATE,
 			  PR_IRUSR | PR_IWUSR);
@@ -1464,16 +1493,17 @@ mok_sign_file (std::string &mok_fingerprint,
 	       const std::string &name,
 	       std::string stapstderr)
 {
-  vector<string> cmd;
   int rc;
   string mok_directory = mok_path + "/" + mok_fingerprint;
 
-  cmd.clear();
-  cmd.push_back (kernel_build_tree + "/scripts/sign-file");
-  cmd.push_back ("sha512");
-  cmd.push_back (mok_directory + MOK_PRIVATE_CERT_FILE);
-  cmd.push_back (mok_directory + MOK_PUBLIC_CERT_FILE);
-  cmd.push_back (name);
+  vector<string> cmd
+    {
+      kernel_build_tree + "/scripts/sign-file",
+      "sha512",
+      mok_directory + MOK_PRIVATE_CERT_FILE,
+      mok_directory + MOK_PUBLIC_CERT_FILE,
+      name
+    };
 
   rc = stap_system (0, cmd);
   if (rc != 0) 
@@ -1494,22 +1524,14 @@ mok_sign_file (std::string &mok_fingerprint,
 static void
 filter_response_file (const string &file_name, const string &responseDirName)
 {
-  vector<string> cmd;
-
   // Filter the server's home directory name
-  cmd.clear();
-  cmd.push_back ("sed");
-  cmd.push_back ("-i");
-  cmd.push_back (string ("s,") + get_home_directory () + ",<server>,g");
-  cmd.push_back (file_name);
+  string swap = string ("s,") + get_home_directory () + ",<server>,g";
+  vector<string> cmd { "sed", "-i", swap, file_name };
   (void) stap_system (0, cmd);
 
   // Filter the server's response directory name
-  cmd.clear();
-  cmd.push_back ("sed");
-  cmd.push_back ("-i");
-  cmd.push_back (string ("s,") + responseDirName + ",<server>,g");
-  cmd.push_back (file_name);
+  swap = string ("s,") + responseDirName + ",<server>,g";
+  cmd = { "sed", "-i", swap, file_name };
   (void) stap_system (0, cmd);
 }
 
@@ -1616,24 +1638,16 @@ generate_mok(string &mok_fingerprint)
   // Actually generate key using openssl.
   public_cert_path = tmpdir + string (MOK_PUBLIC_CERT_FILE);
   private_cert_path = tmpdir + string (MOK_PRIVATE_CERT_FILE);
-  cmd.push_back ("openssl");
-  cmd.push_back ("req");
-  cmd.push_back ("-new");
-  cmd.push_back ("-nodes");
-  cmd.push_back ("-utf8");
-  cmd.push_back ("-sha256");
-  cmd.push_back ("-days");
-  cmd.push_back ("36500");
-  cmd.push_back ("-batch");
-  cmd.push_back ("-x509");
-  cmd.push_back ("-config");
-  cmd.push_back (config_path);
-  cmd.push_back ("-outform");
-  cmd.push_back ("DER");
-  cmd.push_back ("-out");
-  cmd.push_back (public_cert_path);
-  cmd.push_back ("-keyout");
-  cmd.push_back (private_cert_path);
+
+  cmd =
+    {
+      "openssl", "req", "-new", "-nodes", "-utf8",
+      "-sha256", "-days", "36500", "-batch", "-x509",
+      "-config", config_path,
+      "-outform", "DER",
+      "-out", public_cert_path,
+      "-keyout", private_cert_path
+    };
   rc = stap_system (0, cmd);
   if (rc != 0) 
     {
@@ -1662,10 +1676,7 @@ generate_mok(string &mok_fingerprint)
 
 cleanup:
   // Remove the temporary directory.
-  cmd.clear ();
-  cmd.push_back ("rm");
-  cmd.push_back ("-rf");
-  cmd.push_back (tmpdir);
+  cmd = { "rm", "-rf", tmpdir };
   rc = stap_system (0, cmd);
   if (rc != 0)
     server_error (_("Error in tmpdir cleanup"));
@@ -1710,6 +1721,29 @@ handleRequest (const string &requestDirName, const string &responseDirName, stri
   // The name of the translator executable.
   stapargv.push_back ((char *)(getenv ("SYSTEMTAP_STAP") ?: STAP_PREFIX "/bin/stap"));
 
+  // Fetch the target kernel version from request and pass flag to stap
+  string kernel_version;
+  filename = requestDirName + "/sysinfo";
+  ifstream versionfile(filename.c_str());
+  if (versionfile >> kernel_version >> kernel_version) // Skip sysinfo: label
+    {
+      if (kernel_build_tree.find(kernel_version) != kernel_build_tree.end())
+        {
+          stapargv.push_back("-r" + kernel_version);
+        }
+      else
+        {
+          log(_F("Target kernel version %s is not supported by this server",
+              kernel_version.c_str()));
+          return;
+        }
+    }
+  else
+    {
+      server_error(_("Unable to read sysinfo"));
+      return;
+    }
+
   /* Transcribe stap_options.  We use plain wordexp(3), since these
      options are coming from the local trusted user, so malicious
      content is not a concern. */
@@ -1734,6 +1768,8 @@ handleRequest (const string &requestDirName, const string &responseDirName, stri
   stapargv.push_back("--tmpdir=" + new_staptmpdir);
 
   stapargv.push_back ("--client-options");
+
+  string stap_opts = "";
   for (i=1 ; ; i++)
     {
       char stapargfile[PATH_MAX];
@@ -1772,9 +1808,12 @@ handleRequest (const string &requestDirName, const string &responseDirName, stri
 
       arg[st.st_size] = '\0';
       stapargv.push_back (arg);
+      stap_opts.append(arg);
+      stap_opts.append(" ");
       free (arg);
       fclose (argfile);
     }
+  log(_F("Options passed from the client: %s", stap_opts.c_str()));
 
   string stapstdout = responseDirName + "/stdout";
 
@@ -1862,7 +1901,7 @@ handleRequest (const string &requestDirName, const string &responseDirName, stri
 	      // If we signing the module failed, change the staprc to
 	      // 1, so that the client won't try to run the resulting
 	      // module, which wouldn't work.
-	      if (! mok_sign_file (mok_fingerprint, kernel_build_tree,
+	      if (! mok_sign_file (mok_fingerprint, kernel_build_tree[kernel_version],
 				   globber.gl_pathv[0], stapstderr))
 		staprc = 1;
 	    }
@@ -1957,7 +1996,7 @@ handleRequest (const string &requestDirName, const string &responseDirName, stri
       // requires signed modules. The error will have been generated
       // above on the systemtap module itself.
       if (! mok_fingerprint.empty ())
-	mok_sign_file (mok_fingerprint, kernel_build_tree, uprobes_response,
+	mok_sign_file (mok_fingerprint, kernel_build_tree[kernel_version], uprobes_response,
 		       stapstderr);
     }
 
@@ -2055,6 +2094,43 @@ spawn_and_wait (const vector<string> &argv, int *spawnrc,
 #undef CHECKRC
 }
 
+/* Given the path to the compressed request file, return 0 if the size of the
+ * uncompressed request is within the determined limit. */
+int
+check_uncompressed_request_size (const char * zip_file)
+{
+  ostringstream result;
+
+  // Generate the command to heck the uncompressed size
+  vector<string> args { "unzip", "-Zt", zip_file };
+  int rc = stap_system_read (0, args, result);
+  if (rc != 0)
+    {
+    server_error (_F("Unable to check the zipefile size. Error code: %d .", rc));
+    return rc;
+    }
+
+  // Parse the result from the unzip call, looking for the third token
+  vector<string> toks;
+  tokenize(result.str(), toks, " ");
+  if (toks.size() < 3)
+    {
+      // Something went wrong and the format is probably not what we expect.
+      server_error("Unable to check the uncompressed zipfile size. Output came in an unexpected format.");
+      return -1;
+    }
+
+  long uncomp_size = atol(toks[2].c_str());
+  if (uncomp_size < 1 || (unsigned)uncomp_size > max_uncompressed_req_size)
+    {
+      server_error(_F("Uncompressed request size of %ld bytes is not within the expected range of 1 to %zu bytes.",
+                      uncomp_size,max_uncompressed_req_size));
+      return -1;
+    }
+
+  return 0; // If it got to this point, everthing went well.
+}
+
 /* Function:  void *handle_connection()
  *
  * Purpose: Handle a connection to a socket.  Copy in request zip
@@ -2181,13 +2257,16 @@ handle_connection (void *arg)
     }
 #endif
 
+  /* Just before we do any kind of processing, we want to check that the request there will
+   * be enough memory to unzip the file. */
+  if (check_uncompressed_request_size(requestFileName))
+    {
+      goto cleanup;
+    }
+
   /* Unzip the request. */
   secStatus = SECFailure;
-  argv.push_back ("unzip");
-  argv.push_back ("-q");
-  argv.push_back ("-d");
-  argv.push_back (requestDirName);
-  argv.push_back (requestFileName);
+  argv = { "unzip", "-q", "-d", requestDirName, requestFileName };
   rc = stap_system (0, argv);
   if (rc != 0)
     {
@@ -2202,12 +2281,7 @@ handle_connection (void *arg)
 
   /* Zip the response. */
   int ziprc;
-  argv.clear ();
-  argv.push_back ("zip");
-  argv.push_back ("-q");
-  argv.push_back ("-r");
-  argv.push_back (responseFileName);
-  argv.push_back (".");
+  argv = { "zip", "-q", "-r", responseFileName, "." };
   rc = spawn_and_wait (argv, &ziprc, NULL, NULL, NULL, responseDirName);
   if (rc != PR_SUCCESS || ziprc != 0)
     {
@@ -2232,10 +2306,7 @@ cleanup:
 	log (_F("Keeping temporary directory %s", tmpdir));
       else
 	{
-	  argv.clear ();
-	  argv.push_back ("rm");
-	  argv.push_back ("-r");
-	  argv.push_back (tmpdir);
+	  argv = { "rm", "-r", tmpdir };
 	  rc = stap_system (0, argv);
 	  if (rc != 0)
 	    server_error (_("Error in tmpdir cleanup"));
