@@ -270,7 +270,7 @@ mutator::create_process(const string& command)
       return false;
     }
 
-  boost::shared_ptr<mutatee> m(new mutatee(app));
+  auto m = make_shared<mutatee>(app);
   mutatees.push_back(m);
   target_mutatee = m;
   p_target_created = true;
@@ -301,7 +301,7 @@ mutator::attach_process(pid_t pid)
       return false;
     }
 
-  boost::shared_ptr<mutatee> m(new mutatee(app));
+  auto m = make_shared<mutatee>(app);
   mutatees.push_back(m);
   target_mutatee = m;
   p_target_created = false;
@@ -318,7 +318,7 @@ mutator::attach_process(pid_t pid)
 bool
 mutator::init_modoptions()
 {
-  typeof(&stp_global_setter) global_setter = NULL;
+  decltype(&stp_global_setter) global_setter = NULL;
   set_dlsym(global_setter, module, "stp_global_setter", false);
 
   if (global_setter == NULL)
@@ -364,7 +364,7 @@ mutator::init_modoptions()
 void
 mutator::init_session_attributes()
 {
-  typeof(&stp_global_setter) global_setter = NULL;
+  decltype(&stp_global_setter) global_setter = NULL;
   set_dlsym(global_setter, module, "stp_global_setter", false);
 
   if (global_setter == NULL)
@@ -440,8 +440,8 @@ mutator::run_module_init()
     return false;
 
   // First see if this is a shared-memory, multiprocess-capable module
-  typeof(&stp_dyninst_shm_init) shm_init = NULL;
-  typeof(&stp_dyninst_shm_connect) shm_connect = NULL;
+  decltype(&stp_dyninst_shm_init) shm_init = NULL;
+  decltype(&stp_dyninst_shm_connect) shm_connect = NULL;
   set_dlsym(shm_init, module, "stp_dyninst_shm_init", false);
   set_dlsym(shm_connect, module, "stp_dyninst_shm_connect", false);
   if (shm_init && shm_connect)
@@ -467,7 +467,7 @@ mutator::run_module_init()
   // From here, either this is a shared-memory module,
   // or we have no target and thus run init directly anyway.
 
-  typeof(&stp_dyninst_session_init) session_init = NULL;
+  decltype(&stp_dyninst_session_init) session_init = NULL;
   try
     {
       set_dlsym(session_init, module, "stp_dyninst_session_init");
@@ -521,7 +521,7 @@ mutator::run_module_exit()
   // From here, either this is a shared-memory module,
   // or we have no target and thus run exit directly anyway.
 
-  typeof(&stp_dyninst_session_exit) session_exit = NULL;
+  decltype(&stp_dyninst_session_exit) session_exit = NULL;
   try
     {
       set_dlsym(session_exit, module, "stp_dyninst_session_exit");
@@ -553,7 +553,7 @@ mutator::update_mutatees()
 
   for (size_t i = 0; i < mutatees.size();)
     {
-      boost::shared_ptr<mutatee> m = mutatees[i];
+      auto m = mutatees[i];
       if (m != target_mutatee && m->is_terminated())
         {
           mutatees.erase(mutatees.begin() + i);
@@ -655,7 +655,7 @@ int mutator::exit_status ()
   // NB: Only shm modules are new enough to have stp_dyninst_exit_status at
   // all, so we don't need to try in-target for old modules like session_exit.
 
-  typeof(&stp_dyninst_exit_status) get_exit_status = NULL;
+  decltype(&stp_dyninst_exit_status) get_exit_status = NULL;
   set_dlsym(get_exit_status, module, "stp_dyninst_exit_status", false);
   if (get_exit_status)
     {
@@ -669,13 +669,13 @@ int mutator::exit_status ()
 
 
 // Find a mutatee which matches the given process, else return NULL
-boost::shared_ptr<mutatee>
+shared_ptr<mutatee>
 mutator::find_mutatee(BPatch_process* process)
 {
   for (size_t i = 0; i < mutatees.size(); ++i)
     if (*mutatees[i] == process)
       return mutatees[i];
-  return boost::shared_ptr<mutatee>();
+  return {};
 }
 
 
@@ -692,7 +692,7 @@ mutator::dynamic_library_callback(BPatch_thread *thread,
   BPatch_process* process = thread->getProcess();
   staplog(1) << "dlopen \"" << object->name()
              << "\", pid = " << process->getPid() << endl;
-  boost::shared_ptr<mutatee> mut = find_mutatee(process);
+  auto mut = find_mutatee(process);
   if (mut)
     mut->instrument_object_dynprobes(object, targets);
 }
@@ -712,11 +712,11 @@ mutator::post_fork_callback(BPatch_thread *parent, BPatch_thread *child)
   staplog(1) << "post fork, parent " << parent_process->getPid()
 	     << ", child " << child_process->getPid() << endl;
 
-  boost::shared_ptr<mutatee> mut = find_mutatee(parent_process);
+  auto mut = find_mutatee(parent_process);
   if (mut)
     {
       // Clone the mutatee for the new process.
-      boost::shared_ptr<mutatee> m(new mutatee(child_process));
+      auto m = make_shared<mutatee>(child_process);
       mutatees.push_back(m);
       m->copy_forked_instrumentation(*mut);
 
@@ -738,7 +738,7 @@ mutator::exec_callback(BPatch_thread *thread)
 
   staplog(1) << "exec, pid = " << process->getPid() << endl;
 
-  boost::shared_ptr<mutatee> mut = find_mutatee(process);
+  auto mut = find_mutatee(process);
   if (mut)
     {
       // Clear previous instrumentation
@@ -785,7 +785,7 @@ mutator::exit_callback(BPatch_thread *thread,
   int pid = process->getPid();
   staplog(1) << "exit callback, pid = " << pid << endl;
 
-  boost::shared_ptr<mutatee> mut = find_mutatee(process);
+  auto mut = find_mutatee(process);
   if (mut)
     {
       // FIXME: We'd like to call the mutatee's exit_callback()
@@ -833,7 +833,7 @@ mutator::thread_create_callback(BPatch_process *proc, BPatch_thread *thread)
   if (!proc || !thread)
     return;
 
-  boost::shared_ptr<mutatee> mut = find_mutatee(proc);
+  auto mut = find_mutatee(proc);
   if (mut)
     mut->thread_callback(thread, true);
 }
@@ -845,7 +845,7 @@ mutator::thread_destroy_callback(BPatch_process *proc, BPatch_thread *thread)
   if (!proc || !thread)
     return;
 
-  boost::shared_ptr<mutatee> mut = find_mutatee(proc);
+  auto mut = find_mutatee(proc);
   if (mut)
     mut->thread_callback(thread, false);
 }
