@@ -1,5 +1,5 @@
 // stapdyn main program
-// Copyright (C) 2012-2015 Red Hat Inc.
+// Copyright (C) 2012-2017 Red Hat Inc.
 //
 // This file is part of systemtap, and is free software.  You can
 // redistribute it and/or modify it under the terms of the GNU General
@@ -99,8 +99,9 @@ main(int argc, char * const argv[])
 	  break;
 
         case 'V':
+          // PRERELEASE
           printf("Systemtap Dyninst loader/runner (version %s/%s, %s)\n"
-                 "Copyright (C) 2012-2015 Red Hat, Inc. and others\n"
+                 "Copyright (C) 2012-2017 Red Hat, Inc. and others\n"
                  "This is free software; see the source for copying conditions.\n",
                  VERSION, DYNINST_FULL_VERSION, STAP_EXTENDED_VERSION);
           return 0;
@@ -145,11 +146,7 @@ main(int argc, char * const argv[])
   if (!check_dyninst_sebools(pid != 0))
     return 1;
 
-#if __cplusplus >= 201103L /* -std=c++11 */
   unique_ptr<mutator> session(new mutator(module, modoptions));
-#else
-  auto_ptr<mutator> session(new mutator(module, modoptions));
-#endif
   if (!session.get() || !session->load())
     {
       staperror() << "Failed to create the mutator!" << endl;
