@@ -217,7 +217,7 @@ void start_cmd(void)
     /* Close any FDs we still hold, similarly as though this were
        a program being spawned due to an system("") tapset function. */
     closefrom(3);
-    
+
     /* We could call closefrom() here, to make sure we don't leak any
      * fds to the target, but it really isn't needed here since
      * close-on-exec should catch everything. We don't have the
@@ -262,7 +262,7 @@ void start_cmd(void)
 
     dbug(1, "blocking briefly\n");
     alarm(60); /* but not indefinitely */
-    
+
 #if WORKAROUND_BZ467568
     {
       /* Wait for the SIGUSR1 */
@@ -281,6 +281,7 @@ void start_cmd(void)
     raise (SIGCONT); /* Harmless; just passes control to parent. */
 #endif /* !WORKAROUND_BZ467568 */
 
+    alarm(0); /* clear alarms */
     dbug(1, "execing target_cmd %s\n", target_cmd);
 
     /* Note that execvp() is not a direct system call; it does a $PATH
@@ -657,7 +658,7 @@ int stp_main_loop(void)
 
   if (monitor)
       monitor_setup();
-  
+
   /* In monitor mode, we must timeout pselect to poll the monitor
      interface. In non-monitor mode, we must timeout pselect so that
      we can handle pending_interrupts. */
@@ -708,12 +709,9 @@ int stp_main_loop(void)
 	FD_ZERO(&fds);
 	FD_SET(control_channel, &fds);
         maxfd = control_channel;
-	if (monitor) {
-	  FD_SET(STDIN_FILENO, &fds);
-	  FD_SET(monitor_pfd[0], &fds);
-	  if (monitor_pfd[0] > maxfd)
-	    maxfd = monitor_pfd[0];
-	}
+        // Immediately update screen on input
+        if (monitor)
+          FD_SET(STDIN_FILENO, &fds);
 	res = pselect(maxfd + 1, &fds, NULL, NULL, timeout, &mainset);
 	if (res < 0 && errno != EINTR)
 	  {
