@@ -1,5 +1,5 @@
 // parse tree functions
-// Copyright (C) 2005-2015 Red Hat Inc.
+// Copyright (C) 2005-2016 Red Hat Inc.
 //
 // This file is part of systemtap, and is free software.  You can
 // redistribute it and/or modify it under the terms of the GNU General
@@ -99,7 +99,7 @@ probe_point::probe_point (std::vector<component*> const & comps):
 // NB: shallow-copy of compoonents & condition!
 probe_point::probe_point (const probe_point& pp):
   components(pp.components), optional (pp.optional), sufficient (pp.sufficient),
-  well_formed (pp.well_formed), condition (pp.condition)
+  well_formed (pp.well_formed), condition (pp.condition), auto_path (pp.auto_path)
 {
 }
 
@@ -123,7 +123,7 @@ unsigned probe::last_probeidx = 0;
 
 probe::probe ():
   body (0), base (0), tok (0), systemtap_v_conditional (0), privileged (false),
-  id (last_probeidx ++)
+  synthetic (false), id (last_probeidx ++)
 {
 }
 
@@ -134,7 +134,7 @@ probe::probe ():
 probe::probe(probe* p, probe_point* l):
   locations (1, l), body (deep_copy_visitor::deep_copy (p->body)),
   base (p), tok (p->tok), systemtap_v_conditional (p->systemtap_v_conditional),
-  privileged (p->privileged), id (last_probeidx ++)
+  privileged (p->privileged), synthetic (p->synthetic), id (last_probeidx ++)
 {
   assert (p->locals.size() == 0);
   assert (p->unused_locals.size() == 0);
@@ -212,7 +212,8 @@ vardecl::compatible_arity (int a)
 
 
 functiondecl::functiondecl ():
-  body (0), synthetic (false), mangle_oldstyle (false), priority(1)
+  body (0), synthetic (false), mangle_oldstyle (false), has_next(false),
+  priority(1)
 {
 }
 
@@ -342,6 +343,34 @@ string atvar_op::sym_name ()
 }
 
 
+bool
+embedded_expr::tagged_p (const char *tag) const
+{
+  return code.find(tag) != interned_string::npos;
+}
+
+
+bool
+embedded_expr::tagged_p (const interned_string& tag) const
+{
+  return code.find(tag) != interned_string::npos;
+}
+
+
+bool
+embeddedcode::tagged_p (const char *tag) const
+{
+  return code.find(tag) != interned_string::npos;
+}
+
+
+bool
+embeddedcode::tagged_p (const interned_string& tag) const
+{
+  return code.find(tag) != interned_string::npos;
+}
+
+
 // ------------------------------------------------------------------------
 // parse tree printing
 
@@ -543,7 +572,10 @@ void vardecl::print (ostream& o) const
 
 void vardecl::printsig (ostream& o) const
 {
-  o << unmangled_name;
+  if (unmangled_name != "")
+    o << unmangled_name;
+  else
+    o << name;
   if(wrap)
      o << "%";
   if (maxsize > 0)
@@ -595,27 +627,18 @@ embedded_tags_visitor::embedded_tags_visitor(bool all_tags)
     }
 }
 
-bool embedded_tags_visitor::tagged_p (const std::string& tag)
-{
-  return tags.count(tag);
-}
-
-void embedded_tags_visitor::find_tags_in_code (const string& s)
-{
-  set<string>::iterator tag;
-  for (tag = available_tags.begin(); tag != available_tags.end(); ++tag)
-      if(s.find(*tag) != string::npos)
-        tags.insert(*tag);
-}
-
 void embedded_tags_visitor::visit_embeddedcode (embeddedcode *s)
 {
-  find_tags_in_code(s->code);
+  for (auto tag = available_tags.begin(); tag != available_tags.end(); ++tag)
+    if (s->tagged_p(*tag))
+      tags.insert(*tag);
 }
 
 void embedded_tags_visitor::visit_embedded_expr (embedded_expr *e)
 {
-  find_tags_in_code(e->code);
+  for (auto tag = available_tags.begin(); tag != available_tags.end(); ++tag)
+    if (e->tagged_p(*tag))
+      tags.insert(*tag);
 }
 
 void functiondecl::printsigtags (ostream& o, bool all_tags) const
@@ -627,10 +650,8 @@ void functiondecl::printsigtags (ostream& o, bool all_tags) const
   embedded_tags_visitor etv(all_tags);
   this->body->visit(&etv);
 
-  set<string, bool>::const_iterator tag;
-  for (tag = etv.available_tags.begin(); tag != etv.available_tags.end(); ++tag)
-    if (etv.tagged_p(*tag))
-      o << " " << *tag;
+  for (auto tag = etv.tags.begin(); tag != etv.tags.end(); ++tag)
+    o << " " << *tag;
 }
 
 void arrayindex::print (ostream& o) const
@@ -999,6 +1020,7 @@ print_format::string_to_components(string const & str)
 	case 'd':
 	case 'i':
 	  curr.set_flag (fmt_flag_sign);
+	  /* Fallthrough */
 	case 'u':
 	  curr.type = conv_number;
 	  curr.base = 10;
@@ -1011,6 +1033,7 @@ print_format::string_to_components(string const & str)
 
 	case 'X':
 	  curr.set_flag (fmt_flag_large);
+	  /* Fallthrough */
 	case 'x':
 	  curr.type = conv_number;
 	  curr.base = 16;
@@ -1099,11 +1122,19 @@ void stat_op::print (ostream& o) const
       o << "max(";
       break;
 
+    case sc_variance:
+      o << "variance(";
+      break;
+
     case sc_none:
       assert (0); // should not happen, as sc_none is only used in foreach sorts
       break;
     }
   stat->print(o);
+
+  if (ctype == sc_variance && params.size() == 1)
+    o << ", " << params[0];
+
   o << ")";
 }
 
