@@ -1,5 +1,5 @@
 // -*- C++ -*-
-// Copyright (C) 2005-2015 Red Hat Inc.
+// Copyright (C) 2005-2017 Red Hat Inc.
 // Copyright (C) 2006 Intel Corporation.
 //
 // This file is part of systemtap, and is free software.  You can
@@ -11,6 +11,7 @@
 #define STAPTREE_H
 
 #include <map>
+#include <memory>
 #include <stack>
 #include <set>
 #include <string>
@@ -26,16 +27,6 @@ extern "C" {
 #include "util.h"
 #include "stringtable.h"
 
-#if defined(HAVE_TR1_MEMORY)
-#include <tr1/memory>
-using std::tr1::shared_ptr;
-#elif defined(HAVE_BOOST_SHARED_PTR_HPP)
-#include <boost/shared_ptr.hpp>
-using boost::shared_ptr;
-#else
-#error "No shared_ptr implementation found; get boost or modern g++"
-#endif
-
 
 struct token; // parse.h
 struct systemtap_session; // session.h
@@ -133,7 +124,7 @@ struct exp_type_details
   virtual bool expandable() const = 0;
   virtual functioncall *expand(autocast_op* e, bool lvalue) = 0;
 };
-typedef shared_ptr<exp_type_details> exp_type_ptr;
+typedef std::shared_ptr<exp_type_details> exp_type_ptr;
 
 
 struct token;
@@ -188,6 +179,8 @@ struct literal_number: public literal
 struct embedded_expr: public expression
 {
   interned_string code;
+  bool tagged_p (const char *tag) const;
+  bool tagged_p (const interned_string& tag) const;
   void print (std::ostream& o) const;
   void visit (visitor* u);
 };
@@ -347,9 +340,10 @@ struct target_symbol: public expression
 
   interned_string name;
   bool addressof;
+  bool synthetic;
   std::vector<component> components;
   semantic_error* saved_conversion_error; // hand-made linked list
-  target_symbol(): addressof(false), saved_conversion_error (0) {}
+  target_symbol(): addressof(false), synthetic(false), saved_conversion_error (0) {}
   virtual std::string sym_name ();
   void chain (const semantic_error& er);
   void print (std::ostream& o) const;
@@ -396,7 +390,6 @@ struct defined_op: public expression
   void visit (visitor* u);
 };
 
-
 struct entry_op: public expression
 {
   expression *operand;
@@ -548,12 +541,14 @@ enum stat_component_type
     sc_min,
     sc_max,
     sc_none,
+    sc_variance,
   };
 
 struct stat_op: public expression
 {
   stat_component_type ctype;
   expression* stat;
+  std::vector<int64_t> params;
   void print (std::ostream& o) const;
   void visit (visitor* u);
 };
@@ -628,6 +623,7 @@ struct functiondecl: public symboldecl
   statement* body;
   bool synthetic;
   bool mangle_oldstyle;
+  bool has_next;
   int64_t priority;
   functiondecl ();
   void print (std::ostream& o) const;
@@ -664,6 +660,8 @@ std::ostream& operator << (std::ostream& o, const statement& k);
 struct embeddedcode: public statement
 {
   interned_string code;
+  bool tagged_p (const char *tag) const;
+  bool tagged_p (const interned_string& tag) const;
   void print (std::ostream& o) const;
   void visit (visitor* u);
 };
@@ -818,6 +816,7 @@ struct probe_point
   bool sufficient;
   bool well_formed; // used in derived_probe::script_location()
   expression* condition;
+  std::string auto_path;
   void print (std::ostream& o, bool print_extras=true) const;
   probe_point ();
   probe_point(const probe_point& pp);
@@ -841,6 +840,7 @@ struct probe
   std::vector<vardecl*> locals;
   std::vector<vardecl*> unused_locals;
   bool privileged;
+  bool synthetic;
   unsigned id;
 
   probe ();
@@ -1176,7 +1176,9 @@ struct throwing_visitor: public visitor
 
 // A visitor similar to a traversing_visitor, but with the ability to rewrite
 // parts of the tree through require/provide.
-
+//
+// The relaxed_p member variable is cleared if an update occurred.
+//
 struct update_visitor: public visitor
 {
   template <typename T> T* require (T* src, bool clearok=false)
@@ -1204,13 +1206,42 @@ struct update_visitor: public visitor
     values.push(src);
   }
 
-  template <typename T> void replace (T*& src, bool clearok=false)
+  template <typename T> void abort_provide (T* src)
   {
-    src = require(src, clearok);
+    values.push(src);
+    aborted_p = true;
   }
 
+  template <typename T> void replace (T*& src, bool clearok=false)
+  {
+    if (! aborted_p)
+      {
+        const T* old_src = src;
+        T* new_src = require(src, clearok);
+        if (old_src != new_src)
+          {
+            #if 0 /* XXX: this should be parametrized from session.verbose, and
+                     could take place of all of those 'elided ...' messages
+                     in elaborate.cxx optimization visitors. */
+            std::cout << "replaced ";
+            old_src->print(std::cout);
+            std::cout << " with ";
+            new_src->print(std::cout);
+            std::cout << std::endl;
+            #endif
+            relaxed_p = false;
+          }
+        src = new_src;
+      }
+  }
+
+  update_visitor(): aborted_p(false), relaxed_p(true) {}
   virtual ~update_visitor() { assert(values.empty()); }
 
+  // Permit reuse of the visitor object. 
+  virtual void reset() { aborted_p = false; relaxed_p = true; }
+  virtual bool relaxed() { return relaxed_p; }
+    
   virtual void visit_block (block *s);
   virtual void visit_try_block (try_block *s);
   virtual void visit_embeddedcode (embeddedcode *s);
@@ -1255,6 +1286,9 @@ struct update_visitor: public visitor
 
 private:
   std::stack<visitable *> values;
+protected:
+  bool aborted_p;
+  bool relaxed_p;
 };
 
 // A visitor which performs a deep copy of the root node it's applied
@@ -1316,11 +1350,9 @@ struct deep_copy_visitor: public update_visitor
 
 struct embedded_tags_visitor: public traversing_visitor
 {
-  std::set<std::string> available_tags;
-  std::set<std::string> tags; // set of the tags that appear in the code
+  std::set<interned_string> available_tags;
+  std::set<interned_string> tags; // set of the tags that appear in the code
   embedded_tags_visitor(bool all_tags);
-  bool tagged_p (const std::string &tag);
-  void find_tags_in_code (const std::string& s);
   void visit_embeddedcode (embeddedcode *s);
   void visit_embedded_expr (embedded_expr *e);
 };
