@@ -1,5 +1,5 @@
 // tapset for procfs
-// Copyright (C) 2005-2014 Red Hat Inc.
+// Copyright (C) 2005-2017 Red Hat Inc.
 // Copyright (C) 2005-2007 Intel Corporation.
 //
 // This file is part of systemtap, and is free software.  You can
@@ -40,10 +40,15 @@ struct procfs_derived_probe: public derived_probe
   bool target_symbol_seen;
   int64_t maxsize_val;
   int64_t umask; 
+  string variable_name;
 
 
   procfs_derived_probe (systemtap_session &, probe* p, probe_point* l, string ps, bool w, int64_t m, int64_t umask); 
   void join_group (systemtap_session& s);
+
+  // Set up this procfs probe to use a static C variable as input
+  // instead using the probe body.
+  void use_internal_buffer(const std::string& var);
 };
 
 
@@ -79,11 +84,9 @@ public:
 
 struct procfs_var_expanding_visitor: public var_expanding_visitor
 {
-  procfs_var_expanding_visitor(systemtap_session& s, const string& pn,
+  procfs_var_expanding_visitor(systemtap_session& s,
                                string path, bool write_probe);
 
-  systemtap_session& sess;
-  string probe_name;
   string path;
   bool write_probe;
   bool target_symbol_seen;
@@ -99,8 +102,8 @@ procfs_derived_probe::procfs_derived_probe (systemtap_session &s, probe* p,
     maxsize_val(m), umask(umask) 
 {
   // Expand local variables in the probe body
-  procfs_var_expanding_visitor v (s, name, path, write); 
-  v.replace (this->body);
+  procfs_var_expanding_visitor v (s, path, write);
+  var_expand_const_fold_loop (s, this->body, v);
   target_symbol_seen = v.target_symbol_seen;
 }
 
@@ -115,14 +118,7 @@ procfs_derived_probe::join_group (systemtap_session& s)
       // Make sure 'struct _stp_procfs_data' is defined early.
       embeddedcode *ec = new embeddedcode;
       ec->tok = NULL;
-      ec->code = string("struct _stp_procfs_data {\n")
-	  + string("  char *buffer;\n")
-	  + string("  size_t bufsize;\n")
-	  + string("  size_t count;\n")
-	  + string("};\n")
-	  + string("#ifndef STP_PROCFS_BUFSIZE\n")
-	  + string("#define STP_PROCFS_BUFSIZE MAXSTRINGLEN\n")
-	  + string("#endif\n");
+      ec->code = string("#include \"procfs-probes.h\"");
       s.embeds.push_back(ec);
     }
   s.procfs_derived_probes->enroll (this);
@@ -130,6 +126,15 @@ procfs_derived_probe::join_group (systemtap_session& s)
 }
 
 
+void
+procfs_derived_probe::use_internal_buffer(const std::string& var)
+{
+    // We're going to mangle this procfs probe to read from an
+    // internal buffer instead of executing the probe body.
+    variable_name = var;
+}
+
+
 void
 procfs_derived_probe_group::enroll (procfs_derived_probe* p)
 {
@@ -234,30 +239,46 @@ procfs_derived_probe_group::emit_module_decls (systemtap_session& s)
       s.op->newline() << "{";
       s.op->line() << " .path=" << lex_cast_qstring (it->first) << ",";
 
-      if (pset->read_probe != NULL)
-        s.op->line() << " .read_probe=" << common_probe_init (pset->read_probe) << ",";
-
-      if (pset->write_probe != NULL)
-        s.op->line() << " .write_probe=" << common_probe_init (pset->write_probe) << ",";
-
-      s.op->line() << " .buffer=stap_procfs_probe_buffers.buf_" << buf_index++ << ",";
-      if (pset->read_probe != NULL)
-	{
-	  if (pset->read_probe->maxsize_val == 0)
-	    s.op->line() << " .bufsize=STP_PROCFS_BUFSIZE,";
-	  else
-	    s.op->line() << " .bufsize="
-			 << pset->read_probe->maxsize_val << ",";
+      if (pset->read_probe && !pset->read_probe->variable_name.empty())
+        {
+	  s.op->line() << " .buffer=(char *)"
+		       << pset->read_probe->variable_name << ",";
+	  s.op->line() << " .bufsize=sizeof("
+		       << pset->read_probe->variable_name << "),";
+	  s.op->line() << " .count=(sizeof("
+		       << pset->read_probe->variable_name << ") - 1),";
+	  s.op->line() << " .permissions=0444,";
 	}
       else
-	s.op->line() << " .bufsize=MAXSTRINGLEN,";
+        {
+	  if (pset->read_probe != NULL)
+	    s.op->line() << " .read_probe="
+			 << common_probe_init (pset->read_probe) << ",";
 
-       s.op->line() << " .permissions=" << (((pset->read_probe ? 0444 : 0) 
-					 | (pset->write_probe ? 0222 : 0)) &~ 
-					   ((pset->read_probe ? pset->read_probe->umask : 0) 
-					 | (pset->write_probe ? pset->write_probe->umask : 0))) 
-					<< ",";
+	  if (pset->write_probe != NULL)
+	    s.op->line() << " .write_probe=" 
+			 << common_probe_init (pset->write_probe) << ",";
 
+	  s.op->line() << " .buffer=stap_procfs_probe_buffers.buf_"
+		       << buf_index++ << ",";
+	  if (pset->read_probe != NULL)
+	    {
+	      if (pset->read_probe->maxsize_val == 0)
+		s.op->line() << " .bufsize=STP_PROCFS_BUFSIZE,";
+	      else
+		s.op->line() << " .bufsize="
+			     << pset->read_probe->maxsize_val << ",";
+	    }
+	  else
+	    s.op->line() << " .bufsize=MAXSTRINGLEN,";
+
+	  s.op->line() << " .permissions="
+		       << (((pset->read_probe ? 0444 : 0) 
+			    | (pset->write_probe ? 0222 : 0)) &~ 
+			   ((pset->read_probe ? pset->read_probe->umask : 0) 
+			    | (pset->write_probe ? pset->write_probe->umask : 0))) 
+		       << ",";
+	}
       s.op->line() << " },";
     }
   s.op->newline(-1) << "};";
@@ -375,12 +396,14 @@ procfs_derived_probe_group::emit_module_init (systemtap_session& s)
 
   s.op->newline() << "if (spp->read_probe)";
   s.op->newline(1) << "probe_point = spp->read_probe->pp;";
-  s.op->newline(-1) << "else";
+  s.op->newline(-1) << "else if (spp->write_probe)";
   s.op->newline(1) << "probe_point = spp->write_probe->pp;";
+  s.op->newline(-1) << "else";
+  s.op->newline(1) << "probe_point = \"internal buffer\";";
   s.op->indent(-1);
 
   s.op->newline() << "_spp_init(spp);";
-  s.op->newline() << "rc = _stp_create_procfs(spp->path, i, &_stp_proc_fops, spp->permissions, spp);";
+  s.op->newline() << "rc = _stp_create_procfs(spp->path, &_stp_proc_fops, spp->permissions, spp);";
 
   s.op->newline() << "if (rc) {";
   s.op->newline(1) << "_stp_close_procfs();";
@@ -410,10 +433,9 @@ procfs_derived_probe_group::emit_module_exit (systemtap_session& s)
 
 
 procfs_var_expanding_visitor::procfs_var_expanding_visitor (systemtap_session& s,
-							    const string& pn,
 							    string path,
 							    bool write_probe):
-  sess (s), probe_name (pn), path (path), write_probe (write_probe),
+  var_expanding_visitor (s), path (path), write_probe (write_probe),
   target_symbol_seen (false)
 {
   // procfs probes can also handle '.='.
@@ -453,13 +475,13 @@ procfs_var_expanding_visitor::visit_target_symbol (target_symbol* e)
       embeddedcode *ec = new embeddedcode;
       ec->tok = e->tok;
 
-      string fname;
+      string fname = "__private_" + detox_path(string(e->tok->location.file->name));
       string locvalue = "CONTEXT->ips.procfs_data";
 
       if (! lvalue)
         {
-          fname = "_procfs_value_get";
-          ec->code = string("    struct _stp_procfs_data *data = (struct _stp_procfs_data *)(") + locvalue + string("); /* pure */\n")
+          fname += "_procfs_value_get";
+          ec->code = string("    struct _stp_procfs_data *data = (struct _stp_procfs_data *)(") + locvalue + string("); /* pure */ /* stable */\n")
 
             + string("    if (!_stp_copy_from_user(STAP_RETVALUE, data->buffer, data->count))\n")
             + string("      STAP_RETVALUE[data->count] = '\\0';\n")
@@ -470,7 +492,7 @@ procfs_var_expanding_visitor::visit_target_symbol (target_symbol* e)
         {
           if (*op == "=")
             {
-              fname = "_procfs_value_set";
+              fname += "_procfs_value_set";
               ec->code = string("struct _stp_procfs_data *data = (struct _stp_procfs_data *)(") + locvalue + string(");\n")
                 + string("    strlcpy(data->buffer, STAP_ARG_value, data->bufsize);\n")
                 + string("    if (strlen(STAP_ARG_value) > data->bufsize-1)\n")
@@ -480,7 +502,7 @@ procfs_var_expanding_visitor::visit_target_symbol (target_symbol* e)
             }
           else if (*op == ".=")
             {
-              fname = "_procfs_value_append";
+              fname += "_procfs_value_append";
               ec->code = string("struct _stp_procfs_data *data = (struct _stp_procfs_data *)(") + locvalue + string(");\n")
                 + string("    strlcat(data->buffer, STAP_ARG_value, data->bufsize);\n")
                 + string("    if (data->count + strlen(STAP_ARG_value) > data->bufsize-1)\n")
@@ -497,7 +519,7 @@ procfs_var_expanding_visitor::visit_target_symbol (target_symbol* e)
         }
       fname += lex_cast(++tick);
 
-      fdecl->name = fname;
+      fdecl->unmangled_name = fdecl->name = fname;
       fdecl->body = ec;
       fdecl->type = pe_string;
 
@@ -540,6 +562,8 @@ struct procfs_builder: public derived_probe_builder
                      probe_point * location,
                      literal_map_t const & parameters,
                      vector<derived_probe *> & finished_results);
+
+  virtual string name() { return "procfs builder"; }
 };
 
 
