@@ -1,5 +1,5 @@
 // -*- C++ -*-
-// Copyright (C) 2005-2010 Red Hat Inc.
+// Copyright (C) 2005-2017 Red Hat Inc.
 //
 // This file is part of systemtap, and is free software.  You can
 // redistribute it and/or modify it under the terms of the GNU General
@@ -21,7 +21,10 @@ std::vector<derived_probe_group*> all_session_groups(systemtap_session& s);
 std::string common_probe_init (derived_probe* p);
 void common_probe_entryfn_prologue (systemtap_session& s, std::string statestr,
 				    std::string probe, std::string probe_type,
-				    bool overload_processing = true);
+				    bool overload_processing = true,
+				    void (*declaration_callback)(systemtap_session& s, void* data) = NULL,
+				    void (*pre_context_callback)(systemtap_session& s, void* data) = NULL,
+				    void* callback_data = NULL);
 void common_probe_entryfn_epilogue (systemtap_session& s,
 				    bool overload_processing,
 				    bool schedule_work_safe);
@@ -35,6 +38,7 @@ void register_tapset_netfilter(systemtap_session& sess);
 void register_tapset_perf(systemtap_session& sess);
 void register_tapset_utrace(systemtap_session& sess);
 void register_tapset_java(systemtap_session& sess);
+void register_tapset_python(systemtap_session& sess);
 
 std::string path_remove_sysroot(const systemtap_session& sess,
 				const std::string& path);
@@ -58,18 +62,20 @@ public:
 
 struct var_expanding_visitor: public update_visitor
 {
-  static unsigned tick;
-  std::stack<defined_op*> defined_ops;
-  std::set<std::string> valid_ops;
-  interned_string* op;
-
-  var_expanding_visitor ();
+  var_expanding_visitor (systemtap_session& s);
   void visit_assignment (assignment* e);
   void visit_pre_crement (pre_crement* e);
   void visit_post_crement (post_crement* e);
   void visit_delete_statement (delete_statement* s);
   void visit_defined_op (defined_op* e);
 
+protected:
+  systemtap_session& sess;
+  static unsigned tick;
+  std::stack<defined_op*> defined_ops;
+  std::set<std::string> valid_ops;
+  interned_string* op;
+
   void provide_lvalue_call(functioncall* fcall);
 
 private:
