@@ -28,5 +28,5 @@ expect {
     timeout {fail "all pid tests - timeout"}
     eof {fail "all pid tests - unexpected EOF"}
 }
-catch {close}
-wait
+catch { close }
+catch { wait }
