@@ -29,6 +29,24 @@ extern "C" {
 #include <poll.h>
 }
 
+// Sanity check C++11 support.  We're only requiring GCC 4.4's level of
+// functionality, so trust either __cplusplus or the GNUC macros.
+#if !(__cplusplus >= 201103L || \
+  (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 4)))
+#error "C++11 support is required!"
+#endif
+
+// NB: GCC didn't add C++11 final/override until 4.7, but until then it also
+// only had __cplusplus=1 regardless of -std (GCC PR1773).  So checking new
+// __cplusplus is probably good enough here; otherwise we should autoconf it.
+#if __cplusplus >= 201103L
+#define cxx_final final
+#define cxx_override override
+#else
+#define cxx_final
+#define cxx_override
+#endif
+
 #include "privilege.h"
 #include "stringtable.h"
 
@@ -77,7 +95,10 @@ bool is_fully_resolved(const std::string& name,
                        const std::map<std::string,std::string>& sysenv,
                        const std::string& env_path = "PATH");
 const std::string cmdstr_quoted(const std::string& cmd);
+const std::string detox_path(const std::string& str);
 const std::string cmdstr_join(const std::vector<std::string>& cmds);
+const std::string join(const std::vector<std::string>& cmds,
+		       const std::string& delim);
 int stap_waitpid(int verbose, pid_t pid);
 pid_t stap_spawn(int verbose, const std::vector<std::string>& args);
 pid_t stap_spawn(int verbose, const std::vector<std::string>& args,
@@ -91,6 +112,7 @@ inline int stap_system(int verbose, const std::vector<std::string>& args,
                        bool null_out=false, bool null_err=false)
 { return stap_system(verbose, args.front(), args, null_out, null_err); }
 int stap_system_read(int verbose, const std::vector<std::string>& args, std::ostream& out);
+std::pair<bool,int> stap_fork_read(int verbose, std::ostream& out);
 int kill_stap_spawn(int sig);
 void assert_regexp_match (const std::string& name, const std::string& value, const std::string& re);
 int regexp_match (const std::string& value, const std::string& re, std::vector<std::string>& matches);
@@ -119,6 +141,34 @@ inline std::string lex_cast(IN const & in)
   return ss.str();
 }
 
+#if __cplusplus < 201103L
+// Older C++0x only had the "long long" implementations, so we cast up.
+#define INT_TO_STRING(IN) \
+  LEX_CAST_TO_STRING(signed IN, long long) \
+  LEX_CAST_TO_STRING(unsigned IN, unsigned long long)
+#else
+// Otherwise, keep the values native.
+#define INT_TO_STRING(IN) \
+  LEX_CAST_TO_STRING(signed IN, signed IN) \
+  LEX_CAST_TO_STRING(unsigned IN, unsigned IN)
+#endif
+
+#define LEX_CAST_TO_STRING(IN, CAST) \
+  template <> \
+  inline std::string lex_cast(IN const & in) \
+  { \
+    return std::to_string(static_cast<CAST>(in)); \
+  }
+
+INT_TO_STRING(char)
+INT_TO_STRING(short)
+INT_TO_STRING(int)
+INT_TO_STRING(long)
+INT_TO_STRING(long long)
+
+#undef INT_TO_STRING
+#undef LEX_CAST_TO_STRING
+
 
 template <typename OUT>
 inline OUT lex_cast(std::string const & in)
